/*
 * CreateFile.java
 * 
 * Package Version: 4.0
 *   Class Version: 1.0
 *  
 * 		   Project: MEDLAND Project
 * 					Arizona State University
 * 
 * 			Author: Gary R. Mayer
 *   		  Date: 13 February 2008
 * 
 * This file contains the CreateFile class. It is used by InteractionModel (IM)
 * to dynamically create script files to be run in the GRASS environment.
 * 
 * Fixes / Additions:
 * 1.0:
 * 	- None.
 * 
 * Bugs / Issues:
 * 1.0:
 *  - None.
 */

package PenaguilaALv4;

import java.io.*;
import java.util.*;


public class CreateFile
{
	protected String name = null;							// name of the file
	private BufferedWriter output = null;					// buffer writer object
	private File file = null;								// file object
	
	/**
	 * CreateFile - default constructor 
	 * creates a file, called "NewFile", in the current working directory with a header. 
	 * User must call Finalize() when complete.
	 */
	protected CreateFile () throws IOException
	{
		this("." + File.separator, "NewFile", true);
		
	}
	
	/**
	 * CreateFile - constructor 
	 * creates a file in the current working directory with the specified name and adds a header. 
	 * User must call Finalize() when complete.
	 * @param nm (String) - the name of the file
	 */
	protected CreateFile ( String nm ) throws IOException
	{
		this("." + File.separator, nm, true);
		
	}
	
	/**
	 * CreateFile - constructor 
	 * creates a file in the current working directory with the specified name and a header, if requested. 
	 * User must call Finalize() when complete. 
	 * @param nm (string) - the name of the file
	 * @param make_header (boolean) - specifies if a header is added
	 */
	protected CreateFile ( String nm, boolean make_header ) throws IOException
	{
		this("." + File.separator, nm, make_header);
		
	}
	
	/**
	 * CreateFile - constructor
	 * @param path (String) - specifies the explicit path to a folder in which to create the file. 
	 * User must call Finalize() when complete. 
	 * @param nm (String) - the name of the file
	 * @param make_header (boolean) - specifies if a header is added
	 */
	protected CreateFile ( String path, String nm, boolean make_header )
	{
		try
		{
			name = nm;
			
			if ( !path.endsWith(File.separator) )
				path.concat(File.separator);
			
			file = new File(path + name);
			output = new BufferedWriter(new FileWriter(file.toString()));

			if ( make_header )
				InsertHeader();
		}
		
		catch (IOException ioe)
		{
			this.file = null;
			
			try
			{			
				if ( this.output != null )
				{
					output.close();
					this.output = null;
				}
				this.file = null;
			}
			
			catch (IOException ioec)
			{
				System.err.println("Multiple exceptions occurred while creating file : " + path + name + ".\n" + ioe.getMessage() + "\nError closing file.\n" + ioec.getMessage());
			}
			
			System.err.println("An exception occurred while creating file " + path + name + ".\n" + ioe.getMessage());
		}
		
	}
	
	/**
	 * Add() - appends String data to the current line in the file buffer
	 * @param data (String) - the data to append
	 */
	protected void Add ( String data )
	{
		try
		{
			if ( this.output != null )
				output.write(data);
		}
		
		catch (IOException ioe)
		{
			System.err.println("Error writing to file.\n" + ioe.getMessage());
			
			try
			{
				output.close();
			}
			
			catch (IOException ioec)
			{
				System.err.println("Error closing file.\n" + ioec.getMessage());
			}
		}
		
	}
	
	/**
	 * AddLine() - appends String data to the current line in the file buffer and 
	 * automatically inserts a line return.
	 * @param line (String) - the line of data to append
	 */
	protected void AddLine ( String line )
	{
		try
		{
			output.write(line);
			output.newLine();
		}
		
		catch (IOException ioe)
		{
			System.err.println("Error writing to file.\n" + ioe.getMessage());
			
			try
			{
				output.close();
			}
			
			catch (IOException ioec)
			{
				System.err.println("Error closing file.\n" + ioec.getMessage());
			}
		}
		
	}
	
	/**
	 * AddLineReturn() - inserts a line return into the current line in the buffer
	 */
	protected void AddLineReturn ( )
	{
		try
		{
			output.newLine();
		}
		
		catch (IOException ioe)
		{
			System.err.println("Error writing to file.\n" + ioe.getMessage());
			
			try
			{
				output.close();
			}
			
			catch (IOException ioec)
			{
				System.err.println("Error closing file.\n" + ioec.getMessage());
			}
		}
		
	}
	
	/**
	 * WriteFromFile () - writes data from the sourcefile into the current 
	 * location within the new file buffer
	 * @param sourcefile - (String) the name (and possibly location) of the source file 
	 */
	protected void WriteFromFile ( String sourcefile )
	{
		FileReader reader = null;
		int code;
		
		try
		{
			File inFile = new File(sourcefile);
		
			if ( !inFile.exists() )
			{
				IllegalArgumentException iae = new IllegalArgumentException("File, " + sourcefile + ", does not exist.");
				throw iae;
			}

			reader = new FileReader(inFile);
			
			// Read characters until end of file.
			while ( (code = reader.read()) != -1 )
				Add(String.valueOf((char) code));
		}
		
		catch (IOException ioe)
		{
				System.err.println("Error reading source file.\n" + ioe.getMessage());
		}
		
		finally
		{
			try
			{
				reader.close();
			}
			
			catch (IOException ioe)
			{
				System.err.println("Error closing source file.\n" + ioe.getMessage());
			}
		}
		
	}
	
	/**
	 * Finalize() - writes all data in the file buffer to disk. 
	 * This method must be called before the file can be used.
	 */
	protected void Finalize ()
	{
		try
		{
			output.flush();
			output.close();
		}
		
		catch (IOException ioec)
		{
			System.err.println("Error closing file.\n" + ioec.getMessage());
		}
		
	}
	
	/**
	 * Commit() - writes data currently in the file buffer to disk.
	 */
	protected void Commit ()
	{
		try
		{
			output.flush();
		}
		
		catch (IOException ioec)
		{
			System.err.println("Error flushing file buffer.\n" + ioec.getMessage());
		}
		
	}
	
	/**
	 * FullPathName() - retrieves the full path name of the file
	 * @return (String) - the pathname of the file
	 */
	protected String FullPathName ()
	{
		return file.getPath();
		
	}
	
	/**
	 * Delete() - deletes this file
	 */
	protected void Delete ()
	{
		file.delete();
	}
	
	/**
	 * InsertHeader () - inserts a default header into the current location of the file buffer. 
	 * Header includes month and year of creation and amessage about being created by DEVSJAVA.
	 */
	protected void InsertHeader ()
	{
		try
		{
			Calendar now = Calendar.getInstance();
			String date_created = now.get(Calendar.DAY_OF_MONTH) + " ";
			
			switch(now.get(Calendar.MONTH))
			{
			case (Calendar.JANUARY):
				date_created += "Jan ";
			break;
			case (Calendar.FEBRUARY):
				date_created += "Feb ";
			break;
			case (Calendar.MARCH):
				date_created += "Mar ";
			break;
			case (Calendar.APRIL):
				date_created += "Apr ";
			break;
			case (Calendar.MAY):
				date_created += "May ";
			break;
			case (Calendar.JUNE):
				date_created += "Jun ";
			break;
			case (Calendar.JULY):
				date_created += "Jul ";
			break;
			case (Calendar.AUGUST):
				date_created += "Aug ";
			break;
			case (Calendar.SEPTEMBER):
				date_created += "Sep ";
			break;
			case (Calendar.OCTOBER):
				date_created += "Oct ";
			break;
			case (Calendar.NOVEMBER):
				date_created += "Nov ";
			break;
			case (Calendar.DECEMBER):
				date_created += "Dec ";
			break;
			}
			
			date_created += now.get(Calendar.YEAR);
			
			output.write("#!/bin/sh");
			output.newLine();
			output.write("##############################################################");
			output.newLine();
			output.write("#");
			output.newLine();
			output.write("#  This file was automatically generated by DEVSJAVA for");
			output.newLine();
			output.write("#    the MEDLAND Project.");
			output.newLine();
			output.write("#");
			output.newLine();
			output.write("#  Date created: " + date_created);
			output.newLine();
			output.write("#");
			output.newLine();	
			output.write("##############################################################");
			output.newLine();
			output.newLine();
		}
		
		catch (IOException ioe)
		{
			System.err.println("Error writing to file.\n" + ioe.getMessage());
			
			try
			{
				output.close();
			}
			
			catch (IOException ioec)
			{
				System.err.println("Error closing file.\n" + ioec.getMessage());
			}
		}
		
	}
	
}
