/*
 * ExternalExec.java
 * 
 * Package Version: 4.0
 *   Class Version: 1.0
 *  
 * 		   Project: MEDLAND Project
 * 					Arizona State University
 * 
 * 			Author: Gary R. Mayer
 *   		  Date: 13 February 2008
 * 
 * This file contains the ExternalExec class. It is used by the Interaction Model
 * (IM) to dynamically execute script files to be run in the GRASS environment
 * using Java's runtime.exec command. Output from GRASS modules is captured in
 * buffers and parsed.
 * 
 * Fixes / Additions:
 * 1.0:
 * 	- None.
 * 
 * Bugs / Issues:
 * 1.0:
 *  - None.
 */

package PenaguilaALv4;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;


public class ExternalExec
{
	private Runtime rt = null;
	private Process proc = null;
		
	// The first array element is the command to execute. Any additional elements
	// are individual arguments to the command. Arguments normally separated by a
	// space should be placed in separate elements.
	public String [] cmd = null;
	public File directory = null;
	public StreamCapture errorCapture = null;
	public StreamCapture outputCapture = null;
		
	public ExternalExec ( String dir )
	{
		this.directory = new File(dir);
	
	}
		
	public void Run ( String[] cmd )
	{
		this.cmd = cmd;
		
		try
		{
			rt = Runtime.getRuntime();
			proc = rt.exec(this.cmd, null, this.directory);
			
			// Check for error messages
			errorCapture = new StreamCapture(proc.getErrorStream());

			// Check for output
			outputCapture = new StreamCapture(proc.getInputStream());
	
			// Start StreamCapture threads
			errorCapture.start();
			outputCapture.start();
			
			// Wait for thread to finish; then check for errors.
			int exitVal = proc.waitFor();

			if ( exitVal != 0 )
			{
				System.err.println("\nUnknown error in external function call from subroutine Run()");
				return;
			}
		}
		
		catch (Throwable t)
		{
			System.err.println("Error running command.");
			t.printStackTrace();
		}
	}
	
	protected void Done ()
	{
		if ( errorCapture != null )
			errorCapture.Done();
		if ( outputCapture != null )
			outputCapture.Done();

		errorCapture = null;
		outputCapture = null;
		proc = null;
		rt.runFinalization();
		rt.gc();
		
		rt = null;
		
	}
	
}

class StreamCapture extends Thread
{
	private InputStream is;
	private InputStreamReader isr;
	private BufferedReader br;
	
	public StreamCapture ( InputStream is )
	{
		this.is = is;
		this.isr = new InputStreamReader(this.is);
		this.br = new BufferedReader(this.isr);
		
	}
	
	public String GetData () throws IOException
	{
		return GetData(128);
		
	}
	
	public String GetData ( int num_chars ) throws IOException
	{
		String data = null;
		char[] data_buffer = new char[num_chars];
		
		try
		{
			int result = this.br.read(data_buffer, 0, num_chars);
			
			// copy data in buffer if not immediate end of file
			if ( result != -1 )
				data = new String(data_buffer);

		}
		
		catch (IOException ioe)
		{
			data = null;
			throw ioe;
		}
		
		catch (Exception exc)
		{
			System.err.println("ERROR: An exception occurred while reading data from the buffer.");
			System.err.println("Cause: " + exc.getCause());
			System.err.println("Message: " + exc.getMessage());			
		}
		
		finally
		{
			data_buffer = null;
		}
		
		return data;
		
	}
	
	public String GetLine () throws IOException
	{
		String data = null;
		
		try
		{
			data = this.br.readLine();
		}
		
		catch (IOException ioe)
		{
			data = null;
			throw ioe;
		}

		return data;
		 
	}
	
	public void Done ()
	{
		try
		{
			br.close();
			isr.close();
		}
		
		catch (IOException ioe)
		{
			System.err.println("Error closing stream.");
			ioe.getMessage();
		}
	}
	
}
