/*
 * MHousehold.java
 * 
 * Package Version: 4.0
 *   Class Version: 1.0
 * 
 * 		   Project: MEDLAND Project
 * 					Arizona State University
 * 
 * 			Author: Gary R. Mayer & Sean Bergin
 *   		  Date: 05 March 2008
 *     Last Update: July, 2010
 * 
 * This file contains the Household atomic model. It is a single agent that represents a family
 * of humans acting as a cohesive unit. The household interacts with the village manager and
 * the interaction model to achieve its goals.
 * 
 * A household agent is driven by its population. Every person requires some amount of food
 * (kilocalories) per year. Additionally, some or all of the population provides labor (man-days).
 * Food (e.g., wheat) provides some yield per hectare per year (kg/ha/yr). Each type of food
 * provides so many calories per yield (kcal/kg) but requires a certain amount of labor per year
 * to produce that yield (man-days/ha). 
 * 
 * Fixes / Additions:
 * 1.0:
 *  - Incorporated realistic values for wheat farming.
 *  - Modified constructor to accept InitParameters object vice large number of parameters.
 *  - Household now allows dynamic birth rates and only a percentage of the population to
 *  	provide labor.
 * 
 * Bugs / Issues
 * 	(Severity:	1 - LOW: minimal impact to correct execution and/or domain representation, to
 * 				5 - HIGH: causes program crash and/or significant domain representation problems)
 * 1.0:
 *  - (0) .
 */

package PenaguilaALv4;

import java.io.File;
import java.lang.Math;
import java.util.Date;
import java.util.Random;
import java.util.ArrayList;
import model.modeling.message;
import GenCol.entity;
import view.modeling.*;

public class MHousehold extends ViewableAtomic
{
	private static int ticket = -1;		// number to create ID.
	private static Random generator = new Random((new Date()).getTime());
	private final double assess_time = 0.375;
	private final double grazing_plan_time = 0.025;
	private final double manage_time = 0.125;
	private final double update_time = 0.125;
	private float REGION_AGENT_RES = 10.0f;
	private int year;
	
	private Integer myID;					// unique household ID
	private int villageID;					// ID of current village
	private String season;					// the current season; impacts agent actions
	private ArrayList<AgentMsg> LandMgmt;	// outgoing messages to landscape
	private ArrayList<AgentMsg> VMngrRqsts;	// requests to village manager
	private message m;						// messages sent to council and landscape
	private String last_phase;				// last phase before sending output 	
	private String log_path;
	
	///////////////////////////////
	//    SUBSISTENCE VARIABLES  //
	///////////////////////////////
	private boolean limithhSize;
	private int hhSizeLimit;
	private int population;					// number of people in household
	private int init_population;			// initial population
	private double wheat_calories_pc;		// wheat calories required per capita (kcal-wheat/capita/year)
	private int pct_labor;					// percent of population providing labor
	private double labor_pc;				// labor per capita (man-days/capita/year)
	private double pct_prob_birth;				// percent probability for a birth to occur
	private double avg_pct_prob_birth;			// average (and initial) percent probability for a birth
	private int ppl_per_birth_chk;			// number of people per birth check
	private double pct_prob_delta_birth;		// increase / decrease per probability change
	private int min_pct_prob_birth;			// minimum birth probability
	private int max_pct_prob_birth;			// maximum birth probability
	private double pct_prob_death;				// percent probability for a death to occur
	private double avg_pct_prob_death;			// average (and initial) percent probability for a death
	private int ppl_per_death_chk;			// number of people per death check
	private double pct_prob_delta_death;		// increase / decrease per probability change
	private int min_pct_prob_death;			// minimum death probability
	private int max_pct_prob_death;			// maximum death probability
	private int wheatLandNeed;
	private int brlyLandNeed ;
	private int grazedLandNeed; 
	
	///////////////////////////////
	//    FARMING VARIABLES      //
	///////////////////////////////
	private double wheat_kg_pc;				// wheat required per capita (kg-wheat/capita/year)
	private double wheat_init_yield_pha;	// initial wheat field yield per hectare (kg-wheat/ha/year)
	private double wheat_yield;				// wheat field yield (kg-wheat/ha/year)
	private double wheat_labor;				// labor required for wheat (man-days/ha/year)
	private double wheat_calories_per_kg;	// calories from wheat (kcal-wheat/kg-wheat)
	private double seed_percentage; 		//percentage of each harvest which should be devoted to planting in the following year
	private double wheat_expectation;		// in kg, farmed land need factor based upon previous year's yield
	private double barley_expectation;		// in kg, farmed land need factor based upon previous year's yield
	private double exp_yield_scalar;		// yield expectation scalar
	private double wheat_rqmt;				// household wheat requirement (kg)
	private double wheatseed;
	private double barleyseed;
	private double yield_minus_need;		// the wheat yield minus the need (kg-wheat)
	private int max_managed_land;			// maximum land cells the household can manage
	private int max_distance_cost;			// maximum distance cost from village location to land area
	private int init_max_distance_cost;		// initial max distance cost value	
	private double farmed_land_exp;			// the amount of farmed land that is expected to be required
	private float land_per_hectare;			// simulated land cells per hectare

	///////////////////////////////
	//    GRAZING VARIABLES      //
	///////////////////////////////
	private double grazed_land_exp;				// the amount of grazed land that is expected to be required
	private double grazing_impact = 1;				// impact to land cover as a result of farming
	private double total_grazing_fodder_rqmt;			// household sheep/goat total fodder requirement
	private double avg_fodder_return;			// grazing land need factor based upon previous year's yield
	private double grazing_fodder_yield;
	private double init_grazing_expectation = 250; // 
	private double ovicaprid_per_person;
	private double ovicaprid_population;
	private double average_fodder_requirement;// combination of sheep and goat needs, kg/ha
	private int sheep_ratio;
	private int goat_ratio;
	private double ovicaprid_density_factor;
	private int sheep_fodder ;			//kg / year average fodder requirement
	private int goat_fodder ;			//kg / year average fodder requirement
	private double barley_fodder_percentage;
	private boolean fallow_field_grazing;
	private double fodder_from_ff_grazing; // needs to be per /ha
	private double grazing_yield_minus_need;
	private double avg_sheep_caloric_value;
	private double avg_goat_caloric_value;
	private double meat_milk_calories_pc;	// meat and milk calories required per capita (kcal-mm/capita/year)
	private double fodderyieldfromff;
	private double wheat_yield_minus_need;
	private double total_calories_pc;
	private double total_kcal_reqmt;
	private double avg_ovicaprid_caloric_value;
	private double average_fodder_requirement_from_grazing_per_animal;
	private double average_fodder_requirement_from_barley_per_animal;
	private double barley_rqmt;				// household barley requirement which is included as ovicparid fodder (kg)
	private double barley_init_yield_pha;	// initial barley field yield per hectare (kg-barley/ha/year)
	private double barley_yield;			// barley field yield (kg-barley/ha/year)
	// barley labor should just be the same as wheat labor: private double barley_labor; // labor required for barley (man-days/ha/year)

	////////////////////////////////////
	// << DATA GATHERING VARIABLES >> //
	private InfoLog ilog;
	private String mapsetPath;
	private String mapsetName;
	private boolean showinfolog; //= false;
	private boolean popeconlogs;// = true;

	/////////////////////////////////////////////////////
	//                 << METHODS >>                   //	
	public MHousehold()
	{
		this(3, 0, new InitParameters());
		
	}
	
	public MHousehold ( int pop, int vID, InitParameters ip )
	{
		super("    Household_"+ (++ticket) + "    ");
		this.myID = new Integer(ticket);
		this.init_population = pop;
		this.year = ip.date_start;
		this.showinfolog = ip.HouseholdLogOn; // =false;
		this.popeconlogs = ip.popeconlogs;// = true;

		this.sheep_fodder = ip.sheep_fodder;			//kg / year average fodder requirement
		this.goat_fodder = ip.goat_fodder;	 			//kg / year average fodder requirement
		this.avg_sheep_caloric_value = ip.sheep_return;
		this.avg_goat_caloric_value = ip.goat_return; // !! problem
		this.fallow_field_grazing = ip.fallow_field_grazing;
		this.fodder_from_ff_grazing = (2.27 * 5) + 38.64; //50-100kg  y = 2.27x + 38.64    PER HECTARE, 5 is the landcover value
		this.barley_fodder_percentage = ip.barley_fodder_percentage;
		this.limithhSize=ip.hhSizeLimit;
		this.hhSizeLimit=ip.hhSizeExtreme;
		
		this.ovicaprid_per_person = ip.ovicaprid_per_person;
		this.ovicaprid_population = this.init_population * this.ovicaprid_per_person;
		this.sheep_ratio = ip.sheep_ratio;
		this.goat_ratio = ip.goat_ratio;
		int ratio_total = this.sheep_ratio + this.goat_ratio;
		this.average_fodder_requirement = ((this.sheep_fodder * this.sheep_ratio) / ratio_total) + ((this.goat_fodder * this.goat_ratio) / ratio_total);
		this.average_fodder_requirement_from_grazing_per_animal = this.average_fodder_requirement * ((100 - this.barley_fodder_percentage) / 100);
		this.average_fodder_requirement_from_barley_per_animal =  this.average_fodder_requirement * (this.barley_fodder_percentage / 100);
		this.avg_ovicaprid_caloric_value = ((this.avg_sheep_caloric_value * this.sheep_ratio) / ratio_total) + ((this.avg_goat_caloric_value * this.goat_ratio) / ratio_total);
		this.ovicaprid_density_factor = ip.ovicaprid_density;
		this.total_grazing_fodder_rqmt = this.ovicaprid_population * this.average_fodder_requirement;	
		this.grazing_impact = this.grazing_impact * this.ovicaprid_density_factor;
		this.init_grazing_expectation = this.init_grazing_expectation * this.ovicaprid_density_factor;
		this.avg_fodder_return = 0;
		
		this.villageID = vID;
		this.land_per_hectare = 10000.0f / (REGION_AGENT_RES * REGION_AGENT_RES);	// m^2/ha / m^2/land	
		this.init_grazing_expectation = this.init_grazing_expectation / this.land_per_hectare;
		
		this.wheat_calories_pc = ip.wheat_reqd_per_capita;	// kcal-wheat/capita/year
		this.meat_milk_calories_pc = ip.meat_milk_calories_pc;	// kcal-mm/capita/year 
		this.total_calories_pc = this.wheat_calories_pc + this.meat_milk_calories_pc;
		this.pct_labor = ip.labor_pop_pct;
		this.labor_pc = ip.man_days_per_capita;
		this.init_max_distance_cost = ip.maxfarmcost;
		this.avg_pct_prob_birth = ip.birth_init_prob;
		this.ppl_per_birth_chk = ip.birth_prob_group_size;
		this.pct_prob_delta_birth = ip.birth_prob_delta;
		this.min_pct_prob_birth = ip.birth_prob_min;
		this.max_pct_prob_birth = ip.birth_prob_max;
		this.avg_pct_prob_death = ip.death_init_prob;
		this.ppl_per_death_chk = ip.death_prob_group_size;
		this.pct_prob_delta_death = ip.death_prob_delta;
		this.min_pct_prob_death = ip.death_prob_min;
		this.max_pct_prob_death = ip.death_prob_max;
		this.wheat_labor = ip.wheat_labor_reqd;
		this.wheat_calories_per_kg = ip.wheat_calories_provided;
		this.wheat_init_yield_pha = ip.wheat_init_expected_yield;
		this.barley_init_yield_pha = ip.barley_init_expected_yield;
		this.total_kcal_reqmt = this.init_population * this.total_calories_pc ;
		this.seed_percentage = ip.seed_percentage;

		this.mapsetPath = ip.GISDBASE + ip.location  + File.separator;
		this.mapsetName = ip.mapset;

		this.exp_yield_scalar = ip.yield_expecation_scalar;// ??? why is this commented out?
		this.exp_yield_scalar = 100;
		this.init_max_distance_cost = ip.maxfarmcost;

		addInport("info");
		addOutport("vmanager");
		addOutport("ext_village");
		addOutport("land");
		addTestInput("ctrl", new entity("stop"));

	}
	
	public void initialize ()
	{
		this.season = "";
		this.population = this.init_population;
		this.pct_prob_birth = this.avg_pct_prob_birth;
		this.pct_prob_death = this.avg_pct_prob_death;
		this.max_distance_cost = this.init_max_distance_cost;
		this.wheat_expectation = 0.0;										// kg/land
		this.barley_expectation = 0.0;										// kg/land
		this.wheat_yield = 0.0;												// kg-wheat
		this.wheat_kg_pc = this.wheat_calories_pc / this.wheat_calories_per_kg;		// kg-wheat/capita     
		this.wheat_rqmt = this.population * this.wheat_kg_pc;					// kg-wheat
		this.yield_minus_need = 0.0;										// kg
		this.max_managed_land = CalcMaxManagedLandCells();					// land

		LandMgmt = new ArrayList<AgentMsg>();
		VMngrRqsts = new ArrayList<AgentMsg>();		
		generator = new Random((new Date()).getTime());
		
		holdIn("initialize", 0.0);

	}
	
	public void deltext ( double e, message x )
	{
		AgentMsg msg = null;
		Continue(e);
		
		// once dead; stay dead, to prevent farmer zombies
		if ( phaseIs("deceased") )
		{
			ilog.LogInfoLine("Received external event while deceased.");
			return;
		}
		
		if ( phaseIs("stopped") )
		{
			ilog.LogInfoLine("Received external event while stopped.");
			return;
		}
	
		//	EXAMINE, FARM, HARVEST, RELEASE
		try
		{		
			for (int i = 0; i < x.getLength(); i++)
			{
				if ( messageOnPort(x, "info", i) )
				{
					msg = (AgentMsg) x.getValOnPort("info", i);
			
					// results of an examination message
					if ( msg.msgType == Messages.EXAMINE )
					{
						
					}
							
					// review plan success / failure
					else if ( msg.msgType == Messages.PLAN )
					{

					}
					
					// review grazing plan success / failure
					else if ( msg.msgType == Messages.GRAZINGPLAN )
					{

					}
										
					// results of a farming message
					else if ( msg.msgType == Messages.FARM )
					{
						// ignore
					}					
					
					// Calculate current yield from cell responses.
					else if ( msg.msgType == Messages.HARVEST ) // LC*
					{
						ilog.LogInfoLine("Household: deltext(): harvest response msg");
						
						String harvest = (String) msg.Next();
						
						if ( harvest == null )
							throw new IllegalArgumentException("Harvest response message contains no data. ");
						
						while ( harvest != null )
						{
							//System.out.println("HH: returns...."+ this.myID + " "+ harvest);
							//			0			,	1			,	2		,	3		,	4			,5
							//;HH#:avg_wheat_return,avg_barley_return,avgoc_return,wheat_yield,barley_yield,oc_yield;
				 	        String[] sdata = null;
				 	        String dlm_regex = ",";
				 	        sdata = harvest.split(dlm_regex);
				 	       double afr = Double.valueOf(sdata[0]);
				 	       if (afr > 0){
				 	    	  this.wheat_expectation = Double.valueOf(sdata[0]);
				 	       }  
				 	       afr = Double.valueOf(sdata[1]);
				 	       if (afr > 0){
				 	    	  this.barley_expectation = Double.valueOf(sdata[1]);
				 	       }
				 	        afr = Double.valueOf(sdata[2]);
				 	       if (afr > 0){// this prevents agents that do not graze from believing that they cannot get a return if they want to
				 	    	  this.avg_fodder_return = Double.valueOf(sdata[2]); 
				 	       }
				 	       this.wheat_yield = Double.valueOf(sdata[3]);
				 	       this.barley_yield = Double.valueOf(sdata[4]);
						   this.grazing_fodder_yield = Double.valueOf(sdata[5]);; //71000
						   harvest = (String) msg.Next();
						/*	ilog.LogDataLine("harvest","FROM ASSESS:  Year:, "+year+" w_expectation:, " + this.wheat_expectation + " ,b_expectation:, " + this.barley_expectation +", wheat yield=," + wheat_yield + ", barley yield=,"+barley_yield+ ", fodder yield="+this.grazing_fodder_yield +" , avg_fodder, "+ this.avg_fodder_return+ ",time, "+ System.currentTimeMillis() );
							ilog.LogDataLine("grazing","Harvest Message, Year:, "+year+ " ,Pop=," + this.population + ", wheat expectation=," + this.wheat_expectation + ", barley expectation=," + this.barley_expectation + ", previous land need=," + this.farmed_land_exp + ", previous grazing need=,"+ this.grazed_land_exp 
									+", wheat yield=," + this.wheat_yield + ", barley yield=,"+this.barley_yield+ ", fodder yield="+this.grazing_fodder_yield +" , avg_fodder, "+ this.avg_fodder_return );
						*/	
						}		

					}
					
					// results of a release message
					else if ( msg.msgType == Messages.RELEASE )
					{
						//ignore
					}
					
					// release land at the behest of the village
					else if ( msg.msgType == Messages.UNMANAGE )
					{
						ilog.LogInfoLine("Household: deltext(): received unmanage msg");
						String land = (String) msg.Next();
						
					}
					
					else if ( msg.msgType == Messages.CALENDAR )
					{
						msg.ResetPtr();		// all agents share same msg so reset ptr so all get msg.
						this.season = (String) msg.Next();
						//ilog.LogInfoLine("Season Change to:"+ this.season);
					}
				}
				
				else if ( messageOnPort(x, "ctrl", i) )
				{
					String ctrl_msg = (x.getValOnPort("ctrl", i)).toString();
					
					if ( ctrl_msg == "stop" )
					{
						ilog.LogAll("Received STOP command.");
						holdIn("stopped", INFINITY);
					}
				}
			}		
			msg = null; // SS!! memory allocation
		}
		
		catch (IllegalArgumentException iae)
		{
			ilog.LogErrorLine(iae.getMessage());
		}
		
		catch (Exception exc)
		{
			ilog.LogErrorLine("An unknown exception occurred.  " + exc.getMessage());
			exc.printStackTrace();
		}

		holdIn("send", 0.0);
		
	}

	public void deltint()
	{
		try
		{
		if ( phaseIs("initialize") )
		{
			//////////// <<   LOG SETTINGS   >> ///////////////// LOG!
			ilog = new InfoLog();
				File dir = new File(mapsetPath + mapsetName);
				int run_num = 1;
				String old_mapset = mapsetName;
				String new_mapset = null;
				
				if( dir.exists()){
					String run = Integer.toString(run_num);	    
					new_mapset = mapsetName + "_Run" + run;
					dir = new File(mapsetPath + new_mapset);	
					
					while ( dir.exists() ) // still exists, start counting up
					{
						old_mapset = new_mapset;
						run_num++;
						run = Integer.toString(run_num);
						new_mapset = mapsetName + "_Run" + run;
						dir = new File(mapsetPath + new_mapset);		
					}		
				}
				
			    this.log_path = mapsetPath + old_mapset + File.separator + "APSimLogs" + File.separator;
			    dir = new File(this.log_path);
				dir.mkdir();			

				if (this.popeconlogs){
					String[] data_types = {"population", "economy"};// , "troubleshooting" 
					ilog.SetFileInfo(this.log_path, "Household_" + this.myID.intValue() + "_log.txt", data_types);
					ilog.StartAll(InfoLog.DATA, true);
					ilog.SetDisplay(InfoLog.NONE);
				}
			
				if (this.showinfolog){
					String[] data_types = {"general", "population", "economy", "troubleshooting"};// , "troubleshooting" 
					ilog.SetFileInfo(this.log_path, "Household_" + this.myID.intValue() + "_log.txt", data_types);
					ilog.StartAll(InfoLog.DATA + InfoLog.INFO, true);
					ilog.SetDisplay(InfoLog.ALL);
				}
				
			ilog.LogInfoLine("Household: deltint(): initialized");
			ilog.LogDataLine("population", "Year,Pop,Birth %,Death %,Births,Deaths,Yield-Need,Total Kcal Reqmt");
			ilog.LogDataLine("population", year+"," + this.population+","+this.pct_prob_birth+","+this.pct_prob_death+",0,0,"+this.yield_minus_need+","+this.total_kcal_reqmt);
			ilog.LogDataLine("economy","Year,Wheat Rqmt, Wheat Exp, Barley Rqmt, Barley Exp, Farmed Land Exp, Grzd Land Exp, Fodder Exp, Wheat Yield, Barley Yield, Fodder Yield");
					
			PhaseInitialize();
			last_phase = "initialize";
			holdIn("send", 0.0);
		}
		
		else if ( phaseIs("send") )
		{
			if ( last_phase == "initialize" ){
				holdIn("assess", assess_time);
			}

			else if ( last_phase == "assess" ){
				holdIn("update", update_time);
			}
			
			else if ( last_phase == "plan" ){
				holdIn("grazingplan", grazing_plan_time);
			}
			else if ( last_phase == "grazingplan" ){
				ilog.LogDataLine("troubleshooting", " phase is send, lp: gp");
				holdIn("manage", manage_time);
			}
			else if ( last_phase == "manage" ){
				ilog.LogDataLine("troubleshooting", " phase is send, lp: manage");
				holdIn("update", update_time);
			}
			else if ( last_phase == "update" ){
				ilog.LogDataLine("troubleshooting", " phase is send, lp: update");
				holdIn("assess", assess_time);
			}
			else if ( this.population == 0 ){
				holdIn("deceased", INFINITY);
			}
			else{
				holdIn("ERROR", INFINITY);
			}
		}
		
		// Assess last year's cultivation.
		else if ( phaseIs("assess") )
		{
			if ( this.population <= 0 ){
				holdIn("deceased", INFINITY);
				System.out.println("Household "+this.myID+ " is no longer with us...");
			}
			ilog.LogInfoLine("Household: deltint(): assess" + " Season:"+ this.season);
			
			if ( this.season == "spring" ){
				PhaseAssess_FarmLand();
			}
			else if ( this.season == "fall" ){
				++year;
			}
			last_phase = "assess";
		
			if ( HaveOutput() ){
				holdIn("send", 0.0);
			}
			else{
				//holdIn("plan", plan_time);
				holdIn("update", update_time);
			}
			//LC* probably can skip over the other times by going directly to update phase here
		}

		// Cultivate best land from currently managed land and other land that is around. 
		else if ( phaseIs("plan") )
		{
			ilog.LogInfoLine("Household: deltint(): plan_agri"+ " Season:"+ this.season);

			if ( this.season == "spring" ){
				//PhasePlan_FarmLand();
			}
			else if ( this.season == "fall" ) 
			{}

			last_phase = "plan";
			
			if ( HaveOutput() ){
				holdIn("send", 0.0);
			}
			else{
				holdIn("grazingplan", grazing_plan_time);
			}
		}
		
		else if ( phaseIs("grazingplan") )
		{
			ilog.LogInfoLine("Household: deltint(): grazingplan"+ " Season:"+ this.season);

			if ( this.season == "spring" ){
				//PhaseGrazingPlan_GrazingLand();
			}
			else if ( this.season == "fall" ) 
			{}
			last_phase = "grazingplan";
			
			if ( HaveOutput() ){
				holdIn("send", 0.0);
			}
			else{
				holdIn("manage", manage_time);
				}
		}
		
		// Notify land of management decisions.
		else if ( phaseIs("manage") )
		{
			ilog.LogInfoLine("Household: deltint(): manage_land"+ " Season:"+ this.season);

			if ( this.season == "spring" ){
				//PhaseManage_UpdateLandValues();
				//ilog.LogDataLine("troubleshooting", " manage: spring");
				//PhaseManage_FarmLand();
			}
			else if ( this.season == "fall" ){
				//PhaseManage_HarvestFarming();
				//ilog.LogDataLine("troubleshooting", "manage: fall");
			}
			last_phase = "manage";
			
			if ( HaveOutput() ){
				holdIn("send", 0.0);
				//ilog.LogDataLine("troubleshooting", "manage: have output, going to send");
			}
			else{
				holdIn("update", update_time);
				//ilog.LogDataLine("troubleshooting", "manage: no output holding in update");
			}
		}
	
		// Compare yield and need. Modify population, as required.
		else if ( phaseIs("update") )
		{
			ilog.LogInfoLine("Household: deltint(): phase Update"+ " Season:"+ this.season);

			if ( this.season == "fall" )
			{
				PhaseUpdate_UpdatePopulation();
				
			}
			
			last_phase = "update";
			
			if ( HaveOutput() ){
				holdIn("send", 0.0);
			}
			else if ( this.population == 0 ){
				holdIn("deceased", INFINITY);
			}
			else{
				holdIn("assess", assess_time);
				}
		}
	
		else if ( phaseIs("deceased") )
		{
			ilog.LogAll("Household is now DECEASED.");
			holdIn("deceased", INFINITY);
		}
		
		else
		{
			ilog.LogAll("Household " + this.myID.toString() + " entered an invalid phase: " + this.phase + ". Going passive");
			passivate();
		}
		}
		
		catch (Exception exc)
		{
			ilog.LogAll("Unknown exception caught in Hh deltint(). " + exc.getMessage());
			exc.printStackTrace();
		}
		
	}
	
	// Manage external events first.
    public void deltcon(double e, message x)
    {
   		deltext(0, x);
   		deltint();		
    }
	
    // Output function
	public message out()
	{
		AgentMsg lmsg = null, vmmsg = null;
		m = new message();
		
		if ( !LandMgmt.isEmpty() ){
			//ilog.LogInfoLine("Sending land management messages.");
		}
		if ( !VMngrRqsts.isEmpty() ){
			//ilog.LogInfoLine("Sending message to village manager.");
		}
		while ( !LandMgmt.isEmpty() )
		{
			lmsg = (AgentMsg) LandMgmt.remove(0);
			m.add(makeContent("land", lmsg));
		}
		
		while ( !VMngrRqsts.isEmpty() )
		{
			vmmsg = (AgentMsg) VMngrRqsts.remove(0);
			m.add(makeContent("vmanager", vmmsg));
		}
		
	    return m;
	}

	////////////////////////////////////////
	//     Household Phase Functions      //
	////////////////////////////////////////
	private void PhaseInitialize ()
	{		
		// Notify council of initial population.
		AgentMsg pop_msg = new AgentMsg(TypeAgents.HOUSEHOLD, this.myID.intValue(), Messages.POPULATION);
		pop_msg.Add(new Integer(this.population));
		VMngrRqsts.add(pop_msg);
		
		ilog.LogInfoLine("init_pop:" + this.population + ",max_dist_cost:" + this.max_distance_cost);		
	}
	
	private void PhaseAssess_FarmLand ()
	{	
		double bly = 0;
		double wht = 0;
		// Estimate next year's yield and plant more crops if necessary and capable	

		if ( (this.wheat_expectation != 0.0) ){ // kg
			
			this.farmed_land_exp = Math.ceil((this.wheat_rqmt) / (this.wheat_expectation * (this.exp_yield_scalar / 100.0)));
			wht = this.farmed_land_exp;
			bly = Math.ceil((this.barley_rqmt) / (this.barley_expectation * (this.exp_yield_scalar / 100.0)));
			this.farmed_land_exp = this.farmed_land_exp + (Math.ceil((this.barley_rqmt) / (this.barley_expectation * (this.exp_yield_scalar / 100.0))));
		}
		else{ // YEAR 0
			this.wheat_rqmt = this.wheat_rqmt + (this.wheat_rqmt * this.seed_percentage);
			this.farmed_land_exp = Math.ceil(((this.wheat_rqmt) / this.wheat_init_yield_pha) * this.land_per_hectare);
			this.farmed_land_exp = this.farmed_land_exp + Math.ceil(((this.barley_rqmt) / this.barley_init_yield_pha) * this.land_per_hectare);
			wht = Math.ceil(((this.wheat_rqmt) / this.wheat_init_yield_pha) * this.land_per_hectare);
			this.barley_rqmt = this.ovicaprid_population * this.average_fodder_requirement_from_barley_per_animal* (this.barley_fodder_percentage /100) ;//kg
			this.barley_rqmt = this.barley_rqmt + (this.barley_rqmt * this.seed_percentage);
			bly = Math.ceil(((this.barley_rqmt) / this.barley_init_yield_pha) * this.land_per_hectare);
		}

		this.fodderyieldfromff = 0;
		if(this.fallow_field_grazing){
			this.fodderyieldfromff = (this.fodder_from_ff_grazing / this.land_per_hectare) *  (wht + bly); //KG OF FODDER FROM A FARMED HA * NUMBER OF FAMRED CELLS
		}
		
		if ( (this.avg_fodder_return != 0.0 ) ){ 
			this.grazed_land_exp = (this.ovicaprid_population * this.average_fodder_requirement_from_grazing_per_animal) / this.avg_fodder_return;
			ilog.LogInfoLine(" year 2 ovicaprid pop:"+ this.ovicaprid_population + " avg_fodder_req:" + this.average_fodder_requirement_from_grazing_per_animal +" avg_fodder:" + this.avg_fodder_return);
			if(this.fallow_field_grazing){
				// calculating the number of grazing cells that fodder yield is equivalent to
				this.grazed_land_exp = this.grazed_land_exp - (this.fodderyieldfromff / this.avg_fodder_return);
				if (this.grazed_land_exp < 0){
					this.grazed_land_exp = 0;
				}
			}
		}
		else{
			this.grazed_land_exp = (this.ovicaprid_population * this.average_fodder_requirement_from_grazing_per_animal) / this.init_grazing_expectation;
			if (this.grazed_land_exp < 0){
				this.grazed_land_exp = 0;
			}
			ilog.LogInfoLine("grazed land exp:"+this.grazed_land_exp+" year 1 ovicaprid pop:"+ this.ovicaprid_population + " avg_fodder_req:" + this.average_fodder_requirement_from_grazing_per_animal +" init_grazing:" + this.init_grazing_expectation);
			if(this.fallow_field_grazing){
				this.grazed_land_exp = this.grazed_land_exp - (this.fodderyieldfromff / this.init_grazing_expectation);
				if (this.grazed_land_exp < 0){
					this.grazed_land_exp = 0;
				}
			}
		}
		ilog.LogInfoLine("Pop=" + this.population + ", wheat need, " + this.wheat_rqmt+", wheat expectation=" + this.wheat_expectation + ", barley expectation=" + this.barley_expectation + ", land need=" + this.farmed_land_exp + ", grazing need="+ this.grazed_land_exp + ", wheatLandNeed = " + wht + ", brlyLandNeed ="+bly);		
		ilog.LogDataLine("economy", this.year+","+this.wheat_rqmt+","+this.wheat_expectation+","+this.barley_rqmt+","+this.barley_expectation+","+this.farmed_land_exp+","+this.grazed_land_exp+","+this.avg_fodder_return+","+this.wheat_yield+","+this.barley_yield+","+this.grazing_fodder_yield); 
		String LandRequestMsg = null;
		String dlm = ",";

		wheatLandNeed = (int) wht;
		brlyLandNeed  = (int) bly;
		grazedLandNeed = (int) this.grazed_land_exp;
		// hh number, village number, pop, wheat plots, barley plots, grazing plots
		LandRequestMsg = Integer.toString(this.myID )+ dlm + Integer.toString(this.villageID) + dlm + 
		Integer.toString(this.population) + dlm + Integer.toString(wheatLandNeed) + dlm + Integer.toString(brlyLandNeed) + dlm+ Integer.toString(grazedLandNeed);
		System.out.println("LRM:" + LandRequestMsg);
		//LC*
		
		AgentMsg lc_msg = new AgentMsg(TypeAgents.HOUSEHOLD, this.myID.intValue(), Messages.LANDREQUEST, this.villageID);
		if (this.population > 0){
			lc_msg.Add(new String(LandRequestMsg));
		}else{
			holdIn("deceased", INFINITY);
			System.out.println("Household "+this.myID+ " is no longer with us...");
		}
		
		// Get new land views based upon the maximum distance cost that
		//	this household is willing to travel in order to farm
		AgentMsg exm_msg = new AgentMsg(TypeAgents.HOUSEHOLD, this.myID.intValue(), Messages.EXAMINE, this.villageID);
		exm_msg.Add(new Integer(this.max_distance_cost));
		
		if ( !exm_msg.IsEmpty() ){
			LandMgmt.add(lc_msg);
		}
		
	}	
	
	private void PhaseUpdate_UpdatePopulation () // OC*
	{
		ilog.LogInfoLine("PhaseUpdate_UpdatePopulation()");
		double popBuffer = 0.1 * (this.total_kcal_reqmt);		// yield good if within 10% of requirement
		int growth = 0;	
		
		// subtract seed yield from overall yield for wheat and barley
		double wheat_minus_seed = this.wheat_yield - (this.wheat_yield * this.seed_percentage);
		this.barley_yield = this.barley_yield - (this.barley_yield * this.seed_percentage);
		// calculate changes to birth and death percentages
		this.wheat_yield_minus_need = wheat_minus_seed  * this.wheat_calories_per_kg ; //convert to kcal for the whole household
		//     caloric wheat return
		double animals_fed = ((this.barley_yield * 1.25) + this.fodderyieldfromff + this.grazing_fodder_yield) / this.average_fodder_requirement;	
		this.grazing_yield_minus_need = this.avg_ovicaprid_caloric_value * animals_fed; //kcal from ovicaprids

		//	     caloric ovicaprid return
		// total kcal gain or loss for the household 
		this.yield_minus_need =  (this.wheat_yield_minus_need + this.grazing_yield_minus_need) - this.total_kcal_reqmt; 
		//ilog.LogDataLine("harvest", "In Updater: wheat caloric return,"+ this.wheat_yield_minus_need+ ",grazingCaloricReturn,"+this.grazing_yield_minus_need+",yield minus need,"+ this.yield_minus_need+", total kcal need, "+ this.total_kcal_reqmt);	
		if ( Math.abs(this.yield_minus_need) <= popBuffer ) {
			this.yield_minus_need = 0.0;
		}
		// yield was less than the buffered need
		//	increase probability of death, decrease the prob of birth;
		if ( this.yield_minus_need < 0.0 )
		{	
			// if only 15% less than requested	then increase by 1/2 of delta prob
			if( (this.yield_minus_need * -1) < (this.total_kcal_reqmt / 6.6)){  
				this.pct_prob_death = this.pct_prob_death + (this.pct_prob_delta_death / 2); //
				this.pct_prob_birth = this.pct_prob_birth - (this.pct_prob_delta_birth / 2);
			}
			// if only 25% less than requested then increase by 2/3 of delta prob
			else if( (this.yield_minus_need * -1) < (this.total_kcal_reqmt / 4)){  	
				this.pct_prob_death = this.pct_prob_death + ((this.pct_prob_delta_death * 2) / 3);
				this.pct_prob_birth = this.pct_prob_birth - ((this.pct_prob_delta_birth * 2) / 3);
			}
			else{
				this.pct_prob_death += this.pct_prob_delta_death;
				this.pct_prob_birth -= this.pct_prob_delta_birth;
				}
			
			if ( this.pct_prob_death > this.max_pct_prob_death ){
				this.pct_prob_death = this.max_pct_prob_death;
			}
			if ( this.pct_prob_birth < this.min_pct_prob_birth ){
				this.pct_prob_birth = this.min_pct_prob_birth;
			}
		}

		// yield was greater than buffered need
		//	decrease probability of death; increase probability of birth
		else if ( this.yield_minus_need > 0.0 )
		{
			// if only 15% less than requested	then increase by 1/2 of delta prob
			if( (this.yield_minus_need * -1) < (this.total_kcal_reqmt / 6.6)){  
				this.pct_prob_death = this.pct_prob_death - (this.pct_prob_delta_death / 2); //
				this.pct_prob_birth = this.pct_prob_birth + (this.pct_prob_delta_birth / 2);
			}
			// if only 25% less than requested then increase by 2/3 of delta prob
			else if( (this.yield_minus_need * -1) < (this.total_kcal_reqmt / 4)){  	
				this.pct_prob_death = this.pct_prob_death - ((this.pct_prob_delta_death * 2) / 3);
				this.pct_prob_birth = this.pct_prob_birth + ((this.pct_prob_delta_birth * 2) / 3);
			}
			else{
				this.pct_prob_death += this.pct_prob_delta_death;
				this.pct_prob_birth -= this.pct_prob_delta_birth;
				}
			
			if ( this.pct_prob_death < this.min_pct_prob_death ){
				this.pct_prob_death = this.min_pct_prob_death;
			}
			if ( this.pct_prob_birth > this.max_pct_prob_birth ){
				this.pct_prob_birth = this.max_pct_prob_birth;
			}
			//ilog.LogDataLine("growth", "ymn > 0: probdeath:"+ this.pct_prob_death +" prob_birth:"+ this.pct_prob_birth);
			ilog.LogInfoLine("ymn > 0: probdeath:"+ this.pct_prob_death +" prob_birth:"+ this.pct_prob_birth);
		}
		
		// yield was within range of buffered need
		//	settle the death and birth rates toward their average (initial) values
		else
		{
			if ( this.pct_prob_death > this.avg_pct_prob_death )
			{
				this.pct_prob_death -= this.pct_prob_delta_death;
				if ( this.pct_prob_death < this.avg_pct_prob_death ){
					this.pct_prob_death = this.avg_pct_prob_death;
					}
			}
			else if ( this.pct_prob_death < this.avg_pct_prob_death )
			{
				this.pct_prob_death += this.pct_prob_delta_death;
				
				if ( this.pct_prob_death > this.avg_pct_prob_death ){
					this.pct_prob_death = this.avg_pct_prob_death;	
				}
			}
			
			// else, the death percentage is already the average; do nothing
			if ( this.pct_prob_birth > this.avg_pct_prob_birth )
			{
				this.pct_prob_birth -= this.pct_prob_delta_birth;
			
				if ( this.pct_prob_birth < this.avg_pct_prob_birth ){
					this.pct_prob_birth = this.avg_pct_prob_birth;
				}
			}
			
			else if ( this.pct_prob_birth < this.avg_pct_prob_birth )
			{
				this.pct_prob_birth += this.pct_prob_delta_birth;
				
				if ( this.pct_prob_birth > this.avg_pct_prob_birth ){
					this.pct_prob_birth = this.avg_pct_prob_birth;
				}
			}
			//ilog.LogDataLine("growth", "probdeath:,"+ this.pct_prob_death +" ,prob_birth:,"+ this.pct_prob_birth);
			ilog.LogInfoLine("probdeath:,"+ this.pct_prob_death +" ,prob_birth:,"+ this.pct_prob_birth);
			// else, the birth percentage is already the average; do nothing
		}
		
		// determine changes in population
		// NOTE: As households do not currently split as their population grows,
		//	the concept of families within a household is introduced. The birth
		//	and death checks are done for each family within a household.
		int num_births = 0, num_deaths = 0;
		
		// determine if a birth occurs based upon a percentage.
		// as the size of the population increases, another birth may occur
		int num_possible_births = (int) Math.ceil((double) this.population / (double) ppl_per_birth_chk);
		double pct = 0.0;
		for ( int b = 0; b < num_possible_births; b++ )
		{
			pct = generator.nextDouble() * 100.0;
			if ( pct <= pct_prob_birth )
				num_births += 1;
		}
		
		// determine if a death occurs based upon a percentage.
		// as the size of the population increases, another person may die
		int num_possible_deaths = (int) Math.ceil((double) this.population / (double) ppl_per_death_chk);

		for ( int d = 0; d < num_possible_deaths; d++ )
		{
			pct = generator.nextDouble() * 100.0;
			if ( pct <= pct_prob_death )
				num_deaths += 1;
		}
		
		growth = num_births - num_deaths;
	
		
		
				
		ilog.LogInfoLine("Needs: wheat=," + this.wheat_rqmt + ", barley=," + this.barley_rqmt + ", Wheat Yield=," + this.wheat_yield + ", Diff=," + yield_minus_need +
				", wheat_expectation," + this.wheat_expectation + ", barley_expectation," + this.barley_expectation+ ", Pct Probs Death/Birth," + this.pct_prob_death + ", / ," + this.pct_prob_birth + ", Number of deaths," + num_deaths + ", Number of births," + num_births + ", Growth," + growth);


		
		if ( growth != 0 ) 
		{
			this.population += growth;
			if(this.limithhSize){
				if(this.population > this.hhSizeLimit){
					this.population = this.hhSizeLimit;
				}
			}		
			this.wheat_rqmt = (double) this.population * wheat_kg_pc;
			this.wheat_rqmt = this.wheat_rqmt + (this.wheat_rqmt * this.seed_percentage);
			this.total_kcal_reqmt = (double) this.population * this.total_calories_pc;
			this.max_managed_land = CalcMaxManagedLandCells();
			this.ovicaprid_population = this.population *  this.ovicaprid_per_person;
			this.total_grazing_fodder_rqmt = this.ovicaprid_population * this.average_fodder_requirement;
			this.barley_rqmt = this.ovicaprid_population * this.average_fodder_requirement_from_barley_per_animal* (this.barley_fodder_percentage /100) ;//kg
			this.barley_rqmt = this.barley_rqmt + (this.barley_rqmt * this.seed_percentage);
			AgentMsg pop_msg = new AgentMsg(TypeAgents.HOUSEHOLD, this.myID.intValue(), Messages.POPULATION);
			pop_msg.Add(new Integer(this.population));
			VMngrRqsts.add(pop_msg);
			ilog.LogInfoLine("New Pop=" + this.population + ", New Food Rqmt=" + this.wheat_rqmt + " kg wheat and " + this.barley_rqmt + " kg barley, New Max Land=" + this.max_managed_land);
		}
		//ilog.LogDataLine("grazing","UpdatePop, Year:, "+year+ " ,Pop=," + this.population + ", wheat expectation=," + this.wheat_expectation + ", barley expectation=," + this.barley_expectation + ", farmed land need=," +
				//this.farmed_land_exp + ", grazing need=,"+ this.grazed_land_exp+ ",kcal diff," + yield_minus_need +" ,wheatCaloricReturn, "+ this.wheat_yield_minus_need+ 
				//" ,grazingCaloricReturn, "+this.grazing_yield_minus_need+ " ,animals fed,"+animals_fed);
		//ilog.LogInfoLine("YEAR:" + year + ",SEASON:" + this.season + ",POP:" + population + ",prob_birth" + pct_prob_birth + ",prob_death:" + pct_prob_death + ",num_births:" + num_births + ",num_deaths:" + num_deaths + ",growth:" + growth);
		
		if(this.limithhSize){
			if(this.population > this.hhSizeLimit){
				this.population = this.hhSizeLimit;
			}
		}
		ilog.LogDataLine("population", year+"," + this.population+","+this.pct_prob_birth+","+this.pct_prob_death+","+num_births+","+num_deaths+","+this.yield_minus_need+","+this.total_kcal_reqmt);
	}
	
	
	////////////////////////////////////////
	//     Household Helper Functions     //
	////////////////////////////////////////
	public Integer GetID ()
	{
		return myID;
		
	}
	
	public int GetPopulation ()
	{
		return population;
		
	}

	// calculates how many land cells can be managed by the current population
	private int CalcMaxManagedLandCells ()
	{
		//G! this will probably all have to be recalculated
		int max_land_cells = 0;
		
		//  man-days = capita * (laboring capita/total capita (as %)) * man-days/capita 
		float pop_labor = (float) this.population * ((float) this.pct_labor / 100.0f) * (float) this.labor_pc;

		//  man-days/land/yr = man-days/ha/yr / land/ha
		// assume wheat and barley require same amount of work
		float labor_per_land = ((float) this.wheat_labor) / this.land_per_hectare;
		
		//  land/yr = man-days / man-days/land/yr
		max_land_cells = Math.round(pop_labor / labor_per_land);
		
		return max_land_cells;
		
	}
	
	private boolean HaveOutput ()
	{
		if ( !VMngrRqsts.isEmpty() || !LandMgmt.isEmpty() )
			return true;
		
		else
			return false;
		
	}

	////////////////////////////////////////
	//      SimView Helper Functions      //
	////////////////////////////////////////
	public String getTooltipText()
	{
		String stats = "";
		java.text.NumberFormat nf=java.text.NumberFormat.getNumberInstance();
		nf.setMaximumFractionDigits(2);
		
		stats = stats
			+ "\n" + "ID: " + myID
			+ "\n" + "Current season: " + season
			+ "\n" + "Population: " + population
			+ "\n" + "Wheat requirement: " + nf.format(this.wheat_rqmt) + " kg"
			+ "\n" + "Barley requirement: " + nf.format(this.barley_rqmt) + " kg"
			+ "\n" + "Agriculture yield: " + nf.format(this.wheat_yield) + " kg"
			+ "\n" + "Food surplus: " + nf.format(this.yield_minus_need) + " kg"
			+ "\n" + "Expectation: " + nf.format(this.wheat_expectation) + " kg/land"
			+ "\n" + "Expected cultivated land for next year: " + nf.format(farmed_land_exp)
			+ "\n" + "Probability of birth: " + pct_prob_birth
			+ "\n" + "Probability of death: " + pct_prob_death;
		
		return super.getTooltipText() + stats;
	}
}
