/*
 * MHouseholdDemultiplexer.java
 * 
 * Package Version: 2.0
 *   Class Version: 1.1
 * 
 * 		   Project: MEDLAND Project
 * 					Arizona State University
 * 
 * 			Author: Gary R. Mayer
 *   		  Date: 28 June 2007
 * 
 * This file contains the HouseholdDemultiplexer atmoic model. The purpose of this model is
 * intercept all messages going to households within this village. It does this to reduce
 * coupling changes caused by dynamic coupling of hosueholds and villages. 
 * 
 * Note that the model requires that a message be a Pair type with the following format:
 * 	msg = (Integer agent ID, MsgI2A data)
 * 
 * Class Fixes / Additions:
 * 1.1:
 *  - Changed format of received messages from hierarchical-Pair to single-Pair type.
 * 1.0:
 *  - Published version.
 * 0.2:
 * 	- Revised message format to standardized format.
 * 0.1:
 * 		All sub-components are created. Basic messaging from landscape is operational.
 * 
 * Class Bugs / Issues:
 * 1.0:
 *  - None.
 * 0.1:
 * 	- Current scheme requires specifying village ID in message header. This creates too
 * 		tight of a coupling as models (household and cell) sending messages to a household
 * 		must have both the household ID and the current village ID. Recommend modifying schema
 * 		to include only household ID. Local demultipelxers (this model) keep track of local
 * 		households and send messages if household is known. If unknown, message is passed to
 * 		MVillageDemultiplexer which maintains a global cross-reference of households	and villages.
 */

package PenaguilaALv4;

import java.util.*;
import model.modeling.*;
import GenCol.Queue;
import view.modeling.*;


public class MHouseholdDemultiplexer extends ViewableAtomic
{
	private Hashtable Households;
	private Queue MsgQueue;
	
	public MHouseholdDemultiplexer ()
	{
		this("HMsgPasser");
		
	}
	
	public MHouseholdDemultiplexer ( String nm )
	{
		super(nm);
		Households = new Hashtable();
		
		addInport("msg_in");
		addOutport("vmanager");
		// MHousehold output ports added by MVillage

	}
	
	public void initialize ()
	{
		MsgQueue = new Queue();
		
		passivate();
	}
	
	public void deltext ( double e, message x )
	{
		Continue(e);

		for (int i = 0; i < x.getLength(); i++)
		{
			if ( messageOnPort(x, "msg_in", i) )
			{
				AgentMsg msg = (AgentMsg) x.getValOnPort("msg_in", i);
				MsgQueue.addLast((AgentMsg)msg);
			}
		}
		
		if ( !MsgQueue.isEmpty() )
			holdIn("send_msgs", 0);
		
		else
			passivate();
		
	}
	
	public void deltint ()
	{		
		while ( !MsgQueue.isEmpty() )
			holdIn("send_msgs", 0);
		
		passivate();
		
	}
	
	// Manage external events first.
    public void deltcon( double e, message x )
    {
   		deltext(0, x);
   		deltint();
   		
    }
    
	public message out ()
	{
		message m = null;
		AgentMsg msg;
		String port;
		
		m = new message();
		
		while ( !MsgQueue.isEmpty() )
		{
			msg = (AgentMsg) MsgQueue.removeFirst();
			
			// broadcast message; send to all households in this village
			if ( (msg.agentID == -1) && (msg.agentType == TypeAgents.HOUSEHOLD) )
			{
				Enumeration hh_port = Households.elements();
				
				while ( hh_port.hasMoreElements() )
				{
					port = (String) hh_port.nextElement();
					m.add(makeContent(port, msg));
				}
			}
			
			// individual agent message
			else
			{
				port = GetPortFromID(msg.agentType, msg.agentID);
			
				// Send to agent in this village.
				if ( port != null )
					m.add(makeContent(port, msg));
			}	
		}

	    return m;
	    
	}
	
	public String getTooltipText()
	{
		return
			super.getTooltipText()
			+ "\n Number of queued messages: " + MsgQueue.size()
			+ "\n Number of households: " + Households.size();
		
	}
	
	public void AddHousehold ( Integer ID, String port )
	{
		Households.put(ID, port);
		
	}
	
	public void RemoveHousehold ( Integer ID )
	{
		Households.remove(ID);
		
	}
	
	private String GetPortFromID ( TypeAgents agent_type, int agent_id )
	{
		String port = null;
		Integer id = new Integer(agent_id);
		
		if ( agent_type == TypeAgents.VILLAGE )
			port = "vmanager";
		
		else if ( (agent_type == TypeAgents.HOUSEHOLD) && Households.containsKey(id) )
			port = (String) Households.get(id);
		
		return port;
		
	}
}
