/*
 * MInteractionModel.java
 * 
 * Package Version: 4.0
 *   Class Version: 1.0
 *  
 * 		   Project: MEDLAND Project
 * 					Arizona State University
 * 
 * 			Author: Gary R. Mayer & Sean Bergin
 *   		  Date: 11 March 2008
 *     Last Update: July, 2010
 * 
 * This file contains the InteractionModel (IM) atomic model. This model is a
 * model of the interaction between the DEVS agent model and the GRASS landscape
 * model. All communication between the two is handled in this model.
 * 
 * In this version, the landscape is a working landscape model that provides for
 * landscape foliage growth and soil erosion. Foliage is represented by a value
 * between 0 and 50, with 0 representing barren bedrock and 50 representing old
 * forest. Each landscape update cycle, the landscape model improves land that
 * has not been modified by an agent the previous cycle by 1. The soil erosion
 * model is then run using the current foliage values. The agents may then make
 * any changes to the landscape.
 * 
 * Agents require food for survival and growth. To acquire food, agents may
 * farm the land. The agents desire land which is fertile but not heavily
 * populated with brush and tress -- which requires work to clear before farming.
 * However, if given the choice between cleared land with poor soil and that
 * which has trees and shrubs, they will clear the trees and shrubs to farm.
 * 
 * In this version, it is assumed that the landscape and agent models operate
 * using the same update cycle with each cycle equivalent to 1 year. In this
 * version, the IM provides timing for GRASS. As such, it triggers an internal
 * event once per year to run the land cover update and soil erosion models. 
 * 
 * To provide an interface to GRASS, this model sets up a GRASS environment using
 * a pre-generated script file. The IM dynamically creates script files to be
 * executed in the GRASS environment. The scripts are executed using Java's
 * runtime exec and output is captured and parsed from a stream buffer.
 * 
 * The IM performs the following:
 * - Initializes GRASS landscape model and create initial land cover map.
 * - Modifies land cover map with village locations.
 * - Provides villages with location information using central site, area
 * 	 occupied, and land cover.
 * - Creates an initial semi-clearing around new villages.
 * - Provides agents with landscape data.
 * - Maintains a listing of agent impacts against the landscape.
 * - Modifies land cover based on agent actions.
 * - Calculates soil erosion.
 * 
 * Fixes / Additions:
 * 1.0:
 * 	- Initial version.
 * 
 * Bugs / Issues:
 * 	(Severity:	1 - LOW: minimal impact to correct execution and/or domain representation, to
 * 				5 - HIGH: causes program crash and/or significant domain representation problems)
 * 1.0:
 * 	- (*) None.
 */

package PenaguilaALv4;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.*;

import GenCol.Pair;
import model.modeling.message;
import view.modeling.*;
import gov.nasa.worldwind.examples.*;
import grass.GrassFacade;

public class MInteractionModel extends ViewableAtomic
{
	
	/////////////////////////////////
	//  GRASS INTERFACE CONSTANTS  //
	/////////////////////////////////
	private String GRASSPATH = null;				// GRASS data files path
	private String GRASSLOCATION = null;			// GRASS location
	private String GISBASE = null;
	private String GRASSRC6LOCATION = null;
	private String PYTHONLOCATION = null;
	private String MAPSET = null;
	private String REGION_MAP = null;				// used to set region for map operations
	private double REGION_AGENT_RES = 10.0;
	private String max_landcover_map = null;
	
	/////////////////////////////////
	// RECLASSIFICATION RULES      //
    /////////////////////////////////
	private String LANDCOVER_RECLASS_PATH = null;
	private String LANDCOVER_COLOR_PATH = null;
	private String FERTILIY_COLOR_PATH = null;
	private String CFACTOR_RECLASS_PATH = null;
	private String CFACTOR_COLOR_PATH = null;
	
	/////////////////////////////////
	//  R.LAND.ASSESS              //
	/////////////////////////////////
	String LandRequestString = null;
	private double max_wheat_yeld;
	private double max_barley_yeld;
	private double ovicaprid_density_factor;
	private int totalVillages;
	public double degrade_rate;
	private double wooddistweight;
	private double gdistweight;
	private double lcovweight;
	private double fdistweight;
	private double sfertilweight;
	private double sdepthweight;
	private double maxfarmcost;
	private double maxgrazecost;
	private String statsfile_path;
	private String statsfile_name = "stats.txt";
	private String fertil_stats_name = "fertility.txt";
	private int maxlcov;
	private boolean keepRasterMaps;
	private boolean landTenure;
	private int lcBreakPoint1;
	private int lcBreakPoint2;
	private int daBpoint1;
	private int daBpoint2;

	//////////////////////////
	//  STANDARD VARIABLES  //
	//////////////////////////
	private CalendarMgmt cm;
	private double clock;
	private Hashtable AgentResponses;
	private Hashtable VillageSites;
	private TypeMaps[] CellDataMaps;
	private String mapset;
	private String mapsetpath;
	private double last_sigma;
	private boolean harvest_complete = false;
	
	private GrassFacade grass = null;
	private MedFlatWorld medflatworld; // World Wind
	
	private String TEMP_PATH = null; 
	private String USER_PATH = null; 
	private String IMAGES_PATH = null; 
	private int start_year;
	private int year_increment;
	private int end_year;
	private String initial_year = null;
	private InitParameters iparam = null;
	InfoLog ilog;
	private boolean ilog_save;
	private boolean ilog_show;
	private boolean run_quiet;
	
	//LANDSCAPE EVOL PARAMETERS
	private double soil_density ;
	private double r_factor ;
	private String 	climateDataPath;
	private boolean climateDataSeries;
	private double climateData [][];
	private double annual_precip;
	private int storms;
	private double storm_speed;
	private double storm_length;
	private double storm_precip;

	private String stream_transport ;
	private double load_exponent ;
	private String smoothing;
	private double cutoff1 ;
	private double cutoff2 ;
	private double cutoff3 ;
	private int kappa;
	
	private double soil_depth_min; 
	private double soil_depth_max;	
	private int soil_recovery; // soil fertility
	private double wood_pc;
	private double wood_intensity;
	
	// yield calculation, y = a * (x^b), r = 0.982877
	double reg_a = 43.951, reg_b = 0.522071;				// power regression formula values
	private final double land_cycle = 1.0;					// landscape model update frequency
	private double soil_kvalue = 0.0;
	private final int village_clearing_landcover_value = 40;
	private int activeVillages = 0;
	private double friction_value = 0.0;
	private boolean output;
	private boolean WorldWindDisplayOn;

	public MInteractionModel ()
	{
		this("Interactions", null);	
	}
	
	public MInteractionModel ( String name, InitParameters ip )
	{
		super("    " + name + "    ");
		
		// get initialization parameters
		this.iparam = ip;
		this.GRASSPATH = ip.GISDBASE;
		this.GRASSLOCATION = ip.location;
		this.MAPSET = ip.mapset;
		this.REGION_MAP = ip.DEM + "@PERMANENT";
		this.max_landcover_map = ip.max_landcover_map;
		//S LOG
		this.ilog_save = ip.IMLogOn;
		this.ilog_show = false ; //ip.IMLogShow;
		if(this.ilog_show){
			this.run_quiet = false;
		}
		else{this.run_quiet = true;}
		this.keepRasterMaps = ip.KeepRasterMaps;
		
		this.landTenure=ip.landTenure;
		this.lcBreakPoint1=ip.lcBreakPoint1;
		this.lcBreakPoint2=ip.lcBreakPoint2;
		this.daBpoint1=ip.daBpoint1;
		this.daBpoint2=ip.daBpoint2;
						
		LANDCOVER_RECLASS_PATH = ip.landcover_reclass_rules;
		LANDCOVER_COLOR_PATH = ip.landcover_color_rules;
		CFACTOR_RECLASS_PATH = ip.cfactor_reclass_rules;
		CFACTOR_COLOR_PATH = ip.cfactor_color_rules;
		FERTILIY_COLOR_PATH = ip.fertility_color_rules;
		start_year = ip.date_start;
		year_increment = ip.date_delta;
		end_year = ip.date_end;
		this.USER_PATH = ip.APSIMPath;

		this.IMAGES_PATH = USER_PATH + "Images"+ File.separator;	
		this.TEMP_PATH = USER_PATH + "Temp"+ File.separator;
		this.GISBASE = ip.GrassPath;
		this.GRASSRC6LOCATION = ip.grassrc6loc;
		this.PYTHONLOCATION = ip.PythonPath;
		this.output = true; //ip.IMLogOn;
		this.WorldWindDisplayOn = ip.WorldWindDisplayOn;
		
		this.degrade_rate = ip.degrade_rate;
		this.totalVillages = ip.VData.size();
		this.ovicaprid_density_factor = ip.ovicaprid_density;
		this.max_wheat_yeld = ip.maximum_wheat_yield;
		this.max_barley_yeld = ip.maximum_barley_yield;
		this.soil_density = ip.soil_density ;
		this.r_factor = ip.r_factor ;
		this.climateDataPath = ip.climateDataPath;
		this.climateDataSeries = ip.climateDataSeries;
		this.annual_precip = ip.annual_precip;

		this.storms = ip.storms;
		this.storm_speed = ip.storm_speed;
		this.storm_length = ip.storm_length;
		this.stream_transport = ip.stream_transport;
		this.load_exponent = ip.load_exponent;
		this.smoothing = ip.evolSmoothing;
		this.cutoff1  = ip.cutoff1;
		this.cutoff2  = ip.cutoff2;
		this.cutoff3 = ip.cutoff3;
		this.kappa = ip.kappa;
		this.wood_intensity = ip.wood_intensity;
		this.wood_pc = ip.wood_pc;

		this.maxlcov = ip.maxlcov;
		this.soil_depth_min = ip.soil_depth_min;
		this.soil_depth_max = ip.soil_depth_max;	
		this.soil_recovery = ip.soil_recovery;
		this.storm_precip = ip.storm_precip;
		//r.landAssess
		this.wooddistweight = ip.wooddistweight;
		this.gdistweight = ip.gdistweight;
		this.lcovweight = ip.lcovweight;
		this.fdistweight = ip.fdistweight;
		this.sfertilweight = ip.sfertilweight;
		this.sdepthweight = ip.sdepthweight;
		this.maxfarmcost = ip.maxfarmcost;
		this.maxgrazecost = ip.maxgrazecost;
				
		TypeMaps.ELEVATION = new TypeMaps(ip.DEM, true);
		REGION_MAP = TypeMaps.ELEVATION.toString() + "@PERMANENT";
		TypeMaps.LANDCOVER = new TypeMaps(ip.land_cover, true);
		
		if ( ip.friction_map == null )
		{
			this.friction_value = ip.friction_value;
			TypeMaps.FRICTION = null;	
		}
		else{
			TypeMaps.FRICTION = new TypeMaps(ip.friction_map, false);
		}
		if ( ip.soilK_map == null )
		{
			this.soil_kvalue = ip.soilK_value;
			TypeMaps.SOILS_KFACTOR = null;
		}
		else{
			TypeMaps.SOILS_KFACTOR = new TypeMaps(ip.soilK_map, false);
		}
	
		addInport("agent_in");
		addOutport("agent_out");
	}
	
	public void initialize ()
	{
		cm = CalendarMgmt.GetInstance(
				start_year,					// start year
				year_increment,				// year increment 
				5,							// num digits in year
				CalendarMgmt.COUNT_UP,		// increment direction
				"y",						// date prefix
				"_",						// date suffix
				end_year);					// end year	

		AgentResponses = new Hashtable();
		VillageSites = new Hashtable();
		clock = 0.0;
		last_sigma = -1.0;
		initial_year = cm.Year();
		
		// Create a new instance of GRAVA. Grava contains most of the GRASS info, and creates a new Mapset 
		// if the one supplied already exists.
		try {
			/*
			 * Object owner_key, String gisbase, String gisrc, String gisdbase, String location, String mapset						
			 */
			this.grass = GrassFacade.getInstance(this, this.GISBASE, this.GRASSRC6LOCATION, this.GRASSPATH, this.GRASSLOCATION, this.MAPSET); 
			this.grass.GROUP = System.getProperty("user.name");		// this.grass.GROUP = "medland";
			this.MAPSET = this.grass.getMapset();
			mapset = this.MAPSET;
		}
		catch (Exception exc) {
			System.err.println("ERROR DURING GRAVA INITIALIZATION"+exc.getMessage());
		}

		this.grass.setExecutablePath(new File(this.GISBASE + "bin" + File.separator));
		this.grass.setScriptsPath(new File(this.USER_PATH + "Scripts" + File.separator));
		this.grass.setPythonPath(new File (this.PYTHONLOCATION));	

		mapsetpath = GRASSPATH + GRASSLOCATION + File.separator + mapset + File.separator;
		ilog = new InfoLog();
		
		String[] data_type = {"population"};	//"ts",	 "general",  
		if (output){
			File dir = new File(mapsetpath + "APSimLogs");
			dir.mkdir();
			String log_path = mapsetpath + "APSimLogs" + File.separator;
			ilog.SetFileInfo(log_path, "IM_logger.txt", data_type);
			if(this.ilog_save){
				ilog.StartAll(InfoLog.DATA + InfoLog.INFO, true);
			}else{
				ilog.StartAll(InfoLog.DATA, true);
			}
			if(this.ilog_show ){
				ilog.SetDisplay(InfoLog.INFO);
				//ilog.SetDisplay(InfoLog.ERRORS); //InfoLog.ERRORS, 	
			}else{
				ilog.SetDisplay(InfoLog.NONE);
			}
			
			ilog.LogAll("Mapset: " + mapset);
			ilog.LogAll("Year:" + cm.Year());
			ilog.LogInfoLine("IM: deltint(): initialize");
		}	
		
		this.statsfile_path =  mapsetpath + "APSimLogs" + File.separator;

		holdIn("initialize", 0.0);		// initialize GRASS mapset for this simulation
		
	}
	
	// manage external events
	public void deltext ( double e, message x )
	{
		ilog.LogInfoLine(" entering deltext ");
		ArrayList RequestingAgents = null;
		ArrayList LandRequestList = null;
		String LandRequests = null;
		Object data = null;
		Pair rqst = null;
		String lrqst = null; 
		
		AgentMsg agent_msg = null;
		Messages msg_type = null;
		TypeAgents agent_type = null;
		int agent_id = -1;
		Integer village_id = null;
		String msg_key = null;
		
		Continue(e);
		last_sigma = sigma;
		clock = clock + e;
		
		if(x == null){
			holdIn("ERROR", INFINITY);
		}
		
		// an unrecoverable exception occurred; ignore input
		if ( phaseIs("ERROR") ){
			return;
		}
		
		//model has stopped as there are no more agents, or the end year has been reached
		if (phaseIs("PASSIVE") ){
			return;
		}
		
		if (cm.IntYear() >= end_year){
			phaseIs("PASSIVE");
			System.out.println("End year reached");
			ilog.LogInfoLine("IM: deltext(): current phase is " + this.phase + "  End year reached");
			holdIn("PASSIVE", INFINITY);
			System.exit(0);
			return;
		}
		
		ilog.LogInfoLine("IM: deltext(): current phase is " + this.phase);
		
		try
		{
			LandRequestList = new ArrayList();
			
			for (int i = 0; i < x.getLength(); i++)
			{
				if ( messageOnPort(x, "agent_in", i) )
				{
					agent_msg = (AgentMsg) x.getValOnPort("agent_in", i);
	
					// an agent's message, in a single AgentMsg object, is all for the same message (request) type
					msg_type = agent_msg.msgType;
					agent_type = agent_msg.agentType;
					agent_id = agent_msg.agentID;
					village_id = new Integer(agent_msg.locationID);
					msg_key = agent_type.toString() + agent_id + msg_type.toString();
					
					if ( agent_type == TypeAgents.VILLAGE )
					{
						if ( msg_type == Messages.SITE )
						{
							// update the village site
							UpdateSite((VillageData) agent_msg.Next(), msg_key);
							
						}
					}
					
					if (activeVillages==0 || cm.IntYear() >=end_year){
						phaseIs("PASSIVE");
						holdIn("PASSIVE", INFINITY);
						System.out.println("No Active Villages");
						ilog.LogInfoLine("IM: deltext(): current phase is " + this.phase + " No Active Villages");
						holdIn("PASSIVE", INFINITY);
						System.exit(0);
						return;	
					}
					
					// store requests from households in tables in order to efficiently make map queries
					else if ( agent_type == TypeAgents.HOUSEHOLD )
					{
						try
						{
							data = agent_msg.Next();
							
							if ( data == null )
								throw new NullPointerException("Agent " + msg_type.toString() + " message from " + agent_type.toString() + " " + agent_id + " contains no data.");
							
							// examine the surrounding landscape
							// data: an Integer object containing the maximum distance cost to examine
							if ( msg_type == Messages.EXAMINE )
							{
								//System.out.println("Receiving Examine Request");

							}

							else if ( msg_type == Messages.FARM )
							{
								//ilog.LogDataLine("ts", "msg type farm");

							}
							
							else if ( msg_type == Messages.GRAZE )
							{
								//ilog.LogDataLine("ts", "msg type graze");

							}
							//LC*
							else if ( msg_type == Messages.LANDREQUEST )
							{								
								while ( data != null )
								{
									lrqst = (String) data;
									// store new land request	
									LandRequestList.add(lrqst);
									data = agent_msg.Next();
								}

							}
							
							// harvest a particular landscape cell
							// data: a String object containing the cell coordinates [east,north]
							else if ( msg_type == Messages.HARVEST )
							{
								//ilog.LogDataLine("ts", "msg type harvest");
								harvest_complete = true;	
							}
							
							else if ( msg_type == Messages.WOOD )
							{
								
							}
							// add other types of incoming messages here
						}
						
						catch (NullPointerException npe)
						{
							// announce exception and proceed with simulation
							ilog.LogErrorLine(npe.getMessage());
						}
						catch (IllegalArgumentException iae)
						{
							// announce exception and proceed with simulation
							ilog.LogErrorLine(iae.getMessage());
						}
						
						finally
						{
							data = null;
							RequestingAgents = null;
						}
					}
				}
			}

			//LC*
			if ( (LandRequestList != null) && !LandRequestList.isEmpty() )
			{
				//ilog.LogDataLine("ts", "manage tabled examination requests");
				RunLandAssesser(LandRequestList);
				
			}

		}
		
		catch (Exception exc)
		{
			ilog.LogErrorLine(exc.getMessage());
		}
		
		finally
		{
			rqst = null;
			village_id = null;
			agent_type = null;
			msg_type = null;
			data = null;
			agent_msg = null;
			msg_key = null;
			
			if ( RequestingAgents != null )
			{
				RequestingAgents.clear();
				RequestingAgents = null;
			}

		}
		
		ilog.LogInfoLine("Leaving IM: deltext()");
		
		if ( phaseIs("await_sites") ){
			holdIn("initialize", 0.0);
			//ilog.LogDataLine("ts", "phase await sites hold in initialize");
		}
		else if ( !this.AgentResponses.isEmpty() || harvest_complete ){
			holdIn("send2agent", 0.0);
			harvest_complete = false;
			//ilog.LogDataLine("ts", "agent respenses is not empty hold in send2agent");
		}
		else{
			holdIn("update_land", last_sigma);
			//ilog.LogDataLine("ts", "else hold in update land ");
		}
	}
	
	// manage internal transitions
	public void deltint ()
	{

		try
		{
			if ( last_sigma == -1.0 )
				last_sigma = land_cycle;
			// create a new GRASS mapset to hold files for this simulation run
			
			if ( phaseIs("initialize") )
			{ 
				// specify which maps are required for agent data
				SetCellDataMaps();
				System.out.println();
				System.out.println("Mapset: " + mapset+"   Start Year: "+this.start_year+"   End Year: "+this.end_year);
				System.out.println();

				// initialize starting maps
				InitializeLandModel();
				
				// initialize worldwind here		
				if(WorldWindDisplayOn)
				{
					InitializeWW();
				}		
				if (this.climateDataSeries){
					String dlm_regex = ",";
					String data = "";
					String[] sdata = null;
					climateData = new double [6] [end_year+1];
					int row = this.start_year;
					BufferedReader in = new BufferedReader(new FileReader(climateDataPath));
					 try
				        { while ( row < this.end_year+1){ 
				            data = in.readLine();
				            sdata = data.split(dlm_regex);
							climateData [0] [row] =  Double.valueOf(sdata[0]); // storms/yr
							climateData [1] [row] =  Double.valueOf(sdata[1]);// r factor
							climateData [2] [row] =  Double.valueOf(sdata[2]);// annual precip
							climateData [3] [row] =  Double.valueOf(sdata[3]);// storm speed
							climateData [4] [row] =  Double.valueOf(sdata[4]);// storm length
							climateData [5] [row] =  Double.valueOf(sdata[5]);//storm precip
				            row++;
						   }
				        }
					 
				        catch (Exception exc)
				        {
				        	System.err.println(exc.toString());
				            System.out.println("This Error May Result from a mismatch between climate data and the number of simulation years");
				        }
				}
				
				ilog.LogInfoLine("Leaving deltint(): initialize phase");
			}
			
			else if ( phaseIs("update_land") && activeVillages!=0 && cm.IntYear()< end_year+1)
			{
				
				//ilog.LogDataLine("ts", "IM: deltint: update_land");
				Enumeration villages = this.VillageSites.keys();
				String population_line = "year," +cm.Year();
				while ( villages.hasMoreElements() ){
					Integer vID = (Integer) villages.nextElement();
					VillageData vdata = (VillageData) VillageSites.get(vID);
					population_line = population_line + ",village number, " + vdata.villageID + ",pop, " + vdata.population;
				}
				ilog.LogDataLine("population", population_line);
				
				//ilog.LogDataLine("population", "");	
				

				RunLandscapeEvol();		// run erosion model on new land cover
				
				if(WorldWindDisplayOn)
					{
					// update and create new images 
					UpdateWWImages(); // Create the New Images
					medflatworld.UpdateImages(); // Notify WW that the images need to be reloaded
					}
				
				cm.Increment();				// increment calendar year
				System.out.println();
				System.out.println("	******** YEAR: " + cm.Year()+ " ********");
				System.out.println();
				/*Runtime r = Runtime.getRuntime(); // SS!! memory
				r.gc();
				r = null;*/
							
				// reset village site update flags
				villages = this.VillageSites.keys();
				while ( villages.hasMoreElements() )
				{
					Integer vID = (Integer) villages.nextElement();
					VillageData vdata = (VillageData) VillageSites.get(vID);
					vdata.updated = false;
					ilog.LogInfoLine("village number: " + vdata.villageID + " pop: " + vdata.population + " site: "+ vdata.site);
				}
				
				ilog.LogInfoLine("Leaving IM: deltint(): update_land");
			}
			
			if(activeVillages==0 || cm.IntYear() >=end_year+1){
				holdIn("PASSIVE", INFINITY);
			}
			
			// send landscape communication to agents
			else if ( phaseIs("send2agent") )
			{
				//ilog.LogDataLine("ts", "IM: deltint: send to agent");
				// package landscape responses to each agent
			}
			
			if ( !this.AgentResponses.isEmpty() )
			{
				last_sigma = sigma;
				holdIn("send2agent", 0.0);
				//ilog.LogDataLine("ts", "IM: deltint: agent responses is not empty, , being held in send 2 agent");
			}
			
			else
			{
				//ilog.LogDataLine("ts", "IM: deltint: else, update land");
				holdIn("update_land", last_sigma);
				last_sigma = -1.0;
			}
		}
		
		catch (Exception exc)
		{
			ilog.LogErrorLine("An unrecoverable exception occurred during the IM's internal transition.");
			ilog.LogErrorLine(" Current state is: " + this.phase);
			ilog.LogErrorLine(" Cause: " + exc.getCause() + "\n Message: " + exc.getMessage());
			
			holdIn("ERROR", INFINITY);
		}

	}
	
	// manage external events first
    public void deltcon ( double e, message x )
    {
    	//ilog.LogDataLine("ts", "IM: deltcon, ext");
   		deltext(0, x);
   		//ilog.LogDataLine("ts", "IM: deltcon, int");
   		deltint();
    }
    
    // output function
	public message out ()
	{
		Enumeration msg_keys;
		AgentMsg agent_msg = null;
		
		message m = new message();
		
		msg_keys = AgentResponses.keys();

		while ( msg_keys.hasMoreElements() )
		{
			agent_msg = (AgentMsg) this.AgentResponses.remove(msg_keys.nextElement());
			m.add(makeContent("agent_out", agent_msg));
			//ilog.LogInfoLine("Sending " + agent_msg.msgType.toString() + " message to " + agent_msg.agentType.toString() + " " + agent_msg.agentID);
		}

		return m;

	}
	
	private void InitializeWW(){ 	
	// Creates the initial WorldWind images that will be displayed and updated each year. 
	// Also creates the village locations, which does not get updated
		
		CreateWWImages();  
		String out = null;
		try {
			
			char[] flags = {'l', 'g'};
			out = this.grass.gRegion(REGION_MAP, null, flags, -1, -1, false);
			
			
		} catch (IllegalArgumentException e) {
			ilog.LogErrorLine("ERROR: IOException encountered while Initializing WorldWind.");
		} catch (IOException e) {
			ilog.LogErrorLine("ERROR: IOException encountered while Initializing WorldWind.");
		}
		//Parse all of the region extent strings before sending it to WorldWind

		String coord = out;		
		String delims = "nw_lat=";
    	String[] coords = coord.split(delims);
    	delims = "ne_long=";
    	coord = coords[1];
    	coords = coord.split(delims);
     	double max_lat = Double.parseDouble(coords [0]); // nw_lat  
     	coord = out;
     	delims = "se_long=";
     	coords = coord.split(delims);
     	coord = coords[1];
     	delims = "se_lat=";
     	coords = coord.split(delims);
    	double max_long = Double.parseDouble(coords [0]); // se_long
    	coord =out;
    	delims = "sw_long=";
    	coords = coord.split(delims);
    	coord = coords[1]; 
    	delims = "sw_lat=";
    	coords = coord.split(delims);
    	double min_long = Double.parseDouble(coords [0]); // sw_long
    	coord = out; 
    	delims = "sw_lat=";
    	coords = coord.split(delims);
    	coord = coords[1];
    	delims = "center_long=";
    	coords = coord.split(delims);
    	double min_lat = Double.parseDouble(coords [0]); // sw_lat  
    	//System.out.println("sw_lat= "+min_lat +" sw_long: "+ min_long + " se_long: "+max_long+" nw_lat: "+max_lat);

		medflatworld =  new MedFlatWorld(min_lat, max_lat, min_long, max_long, IMAGES_PATH, (cm.IntYear()-1)); 	
		medflatworld.main(null);   
		
		ilog.LogInfoLine("Returning from Initialize WorldWind");
		
	}
	
	private void UpdateWWImages () 
	{
		// Create New Images for WorldWind based on the agent and environment changes
		String last_output = null;
		boolean run_quiet = true;
		
		ilog.LogInfoLine("Updating Images for WorldWind...");
		
		 try {  

	        	last_output = this.grass.rPngoutPy(CurrentMap(TypeMaps.LANDCOVER), IMAGES_PATH+"landcover.png", run_quiet);
	        	ilog.LogInfoLine(last_output);
	        	last_output = this.grass.rPngoutPy(CurrentMap(TypeMaps.SOILDEPTH), IMAGES_PATH+"soil.png", run_quiet);
	        	ilog.LogInfoLine(last_output);
	        	last_output = this.grass.rPngoutPy(CurrentMap(TypeMaps.IMPACTS), IMAGES_PATH+"landuse.png", run_quiet);
	        	ilog.LogInfoLine(last_output);
	        	
	        	//Since there is no net change for the start year,  an elevation map is initially shown.
				if (cm.IntYear() == start_year){
					last_output = this.grass.rPngoutPy(CurrentMap(TypeMaps.ELEVATION), IMAGES_PATH+"cumulative.png", run_quiet);
				}
				else{
					last_output = this.grass.rPngoutPy(CurrentMap(TypeMaps.NETCHANGE), IMAGES_PATH+"cumulative.png", run_quiet);
				}
				ilog.LogInfoLine(last_output);
			
	        }
			   catch (IOException e) {
	     	   ilog.LogErrorLine("An error occurred while updating the WWIMAGES.\n"); 
	        }
			   
		ilog.LogInfoLine("Returning from UpdateWWImages()");
		
	}
	
	private void CreateWWImages () 
	{
		ilog.LogInfoLine("Creating the Initial WorldWind Images");
		String village_textfile = TEMP_PATH+"village_locations.txt";
		String village_vector_map = "village_locations";
		String last_output = null;
		boolean run_quiet = true;
		
		//  create .txt file of the LatLong locations of the villages
		BufferedWriter output;

		try {
            output = new BufferedWriter(new FileWriter(village_textfile));
            Iterator vlg = ModelInitialization.ip.VData.iterator();
            while ( vlg.hasNext() )
			{
				InitParameters.VillageData vdata = (InitParameters.VillageData) vlg.next();
				Integer ns = new Integer((int)vdata.vns);
				Integer ew = new Integer((int)vdata.vew);
    			output.write(ew +"|"+ ns);
    			output.newLine();
    			output.flush();
    		}
    		output.close();
		}
           catch (IOException e) {
        	   ilog.LogErrorLine("An error occurred while writing the village locations to a file.\n"); 
           }
           
        try {  
        	last_output = this.grass.vInASCII(village_textfile, village_vector_map, null, run_quiet);
        	ilog.LogInfoLine(last_output);
        	last_output = this.grass.vPngoutPy(village_vector_map, IMAGES_PATH+"villages.png", run_quiet);
        	ilog.LogInfoLine(last_output);
        	last_output = this.grass.rPngoutPy(CurrentMap(TypeMaps.LANDCOVER), IMAGES_PATH+"landcover.png", run_quiet);
        	ilog.LogInfoLine(last_output);
        	last_output = this.grass.rPngoutPy(CurrentMap(TypeMaps.SOILDEPTH), IMAGES_PATH+"soil.png", run_quiet);
        	ilog.LogInfoLine(last_output);
        	last_output = this.grass.rPngoutPy(CurrentMap(TypeMaps.ELEVATION), IMAGES_PATH+"landuse.png", run_quiet);
        	ilog.LogInfoLine(last_output);
        	last_output = this.grass.rPngoutPy(CurrentMap(TypeMaps.ELEVATION), IMAGES_PATH+"cumulative.png", run_quiet);
        	ilog.LogInfoLine(last_output);
		
        }
		   catch (IOException e) {
     	   ilog.LogErrorLine("An error occurred while creating the intial WW images.\n"); 
        }

		ilog.LogInfoLine("Returning from CreateWWImages()");
		
	}
	
	private void InitializeLandModel () throws IOException, IllegalArgumentException 
	{
		ilog.LogInfoLine("	IM: Running InitializeLandModel()   -Creating Initial Landscape Maps");
		System.out.println();
		System.out.println(" ~	IM: Running InitializeLandModel()    -Creating Initial Landscape Maps");
		System.out.println("IL: "+this.ilog_save);
		try{
			//boolean run_quiet = false;
			String lastresult = null;
			String PermanentMapset = "@PERMANENT";
			char[] flags = null;			
		
			lastresult = this.grass.gRegion(REGION_MAP, null, run_quiet);
			ilog.LogInfoLine(lastresult);
			lastresult = this.grass.gCopyRast(this.iparam.DEM + PermanentMapset, CurrentMap(TypeMaps.ELEVATION), run_quiet);
			ilog.LogInfoLine(lastresult);
			if ( TypeMaps.SOILS_KFACTOR != null )
				lastresult = this.grass.gCopyRast(this.iparam.soilK_map + PermanentMapset, CurrentMap(TypeMaps.SOILS_KFACTOR), run_quiet);
			ilog.LogInfoLine(lastresult);
			if ( TypeMaps.FRICTION != null )
				lastresult = this.grass.gCopyRast(this.iparam.friction_map + PermanentMapset, CurrentMap(TypeMaps.FRICTION), run_quiet);
			ilog.LogInfoLine(lastresult);
			lastresult = this.grass.gCopyRast(this.iparam.fertility_map + PermanentMapset, CurrentMap(TypeMaps.FERTILITY), run_quiet);
			ilog.LogInfoLine(lastresult);
			lastresult = this.grass.gCopyRast(this.iparam.land_cover + PermanentMapset, "temp_" + CurrentMap(TypeMaps.LANDCOVER), run_quiet);
			ilog.LogInfoLine(lastresult);
			
			if(iparam.bedrock_map != null){
				System.out.println("Creating soil map from existing bedrock map");
				lastresult = this.grass.gCopyRast(this.iparam.bedrock_map + PermanentMapset, CurrentMap(TypeMaps.BEDROCK), run_quiet);
				ilog.LogInfoLine(lastresult);	
				lastresult = this.grass.rMapcalc(CurrentMap(TypeMaps.SOILDEPTH) + "="+CurrentMap(TypeMaps.ELEVATION)+ "-"+ CurrentMap(TypeMaps.BEDROCK));
			}
			else{
				System.out.println("Creating soil map with normal procedure");
				lastresult = this.grass.rSoilDepthPy(CurrentMap(TypeMaps.ELEVATION), CurrentMap(TypeMaps.BEDROCK), CurrentMap(TypeMaps.SOILDEPTH), this.soil_depth_min, this.soil_depth_max);
				ilog.LogInfoLine(lastresult);
			}
			
			lastresult = this.grass.rSlopeAspect(CurrentMap(TypeMaps.ELEVATION), CurrentMap(TypeMaps.SLOPE), run_quiet);
			ilog.LogInfoLine(lastresult);
			lastresult = this.grass.rReclass(LANDCOVER_RECLASS_PATH, "temp_" +CurrentMap(TypeMaps.LANDCOVER), "temp_" +CurrentMap(TypeMaps.LANDCOVER)+ "_reclass", null, true);
			ilog.LogInfoLine(lastresult);
			lastresult = this.grass.rMapcalc(CurrentMap(TypeMaps.LANDCOVER) + "=temp_" + CurrentMap(TypeMaps.LANDCOVER) + "_reclass");
			ilog.LogInfoLine(lastresult);
			lastresult = this.grass.gRemoveRast("temp_" + CurrentMap(TypeMaps.LANDCOVER) + "_reclass,temp_" + CurrentMap(TypeMaps.LANDCOVER), run_quiet);
			ilog.LogInfoLine(lastresult);
			lastresult = this.grass.rColors(CurrentMap(TypeMaps.LANDCOVER), flags, GrassFacade.ColorSource.RULES_FILE, LANDCOVER_COLOR_PATH, run_quiet);
			ilog.LogInfoLine(lastresult);
			lastresult = this.grass.rColors(CurrentMap(TypeMaps.FERTILITY), flags, GrassFacade.ColorSource.RULES_FILE, FERTILIY_COLOR_PATH, run_quiet);
			ilog.LogInfoLine(lastresult);			

		} catch (IllegalArgumentException e) {
			ilog.LogErrorLine("ERROR: IOException encountered while Initializing InitializeLandModel.");
		} catch (IOException e) {
			ilog.LogErrorLine("ERROR: IOException encountered while Initializing InitializeLandModel.");
		}
		ilog.LogInfoLine("	Returning from InitializeLandModel()");
		System.out.println();
		System.out.println("~	Returning from InitializeLandModel()");
		System.out.println();
		
	}
	
	private void UpdateSite ( VillageData vdata, String msg_key ) 
	{
		ilog.LogInfoLine("	IM: Running UpdateSite()");
		System.out.println();
		System.out.println("~	IM: Running UpdateSite()");
		System.out.println();
		//boolean run_quiet = false;
		final String vlg_id = "village_" + vdata.villageID + "_";
		final String site_map = CurrentMap(vlg_id, TypeMaps.SITE);
		final String vector_site_location = initial_year+vlg_id+"site@"+mapset;
		final String data_fname = site_map + ".dat";
		//final String site_map_loc = CurrentMapLoc(vlg_id, TypeMaps.SITE);
		final String cost_map = CurrentMap(vlg_id, TypeMaps.COSTSFC);
		final String cost_map_loc = CurrentMapLoc(vlg_id, TypeMaps.COSTSFC);	
		final String village_catchment_map = cm.Year() + vlg_id + "catchment";
		final String village_catchment_map_loc = village_catchment_map + "@" + mapset;	
		final String village_map = CurrentMap(vlg_id, TypeMaps.CLEARING);
		final String village_map_loc = CurrentMapLoc(vlg_id, TypeMaps.CLEARING);
		//final String prev_village_map_loc = cm.PrevYear() + vlg_id + TypeMaps.CLEARING.toString() + "@" + mapset;
		final String elev_map_loc = CurrentMapLoc(TypeMaps.ELEVATION);
		
		String fric_map_loc = null;
		if ( TypeMaps.FRICTION != null ){
			fric_map_loc = CurrentMapLoc(TypeMaps.FRICTION);
		}
		else{fric_map_loc = String.valueOf(this.friction_value);}
		
		final String landcover_map_loc = CurrentMapLoc(TypeMaps.LANDCOVER);
		final String temp_landcover_map = cm.Year() + vlg_id + "temp_" + TypeMaps.LANDCOVER.toString();
		//final String temp_landcover_reclass_map = temp_landcover_map + "_reclass";
		boolean new_site = false;
		CreateFile sitedata = null;
		AgentMsg msg = null;
		String result = null;
		Integer village_id = null;
		String lastoutput = null;
		
		try
		{
			ilog.LogInfoLine("IM received site msg: Village " + vdata.villageID + " is at site " + vdata.site);
			// if the village doesn't exist, create it
			village_id = new Integer(vdata.villageID);		
			if ( !VillageSites.containsKey(village_id) )
			{
				new_site = true;
				activeVillages++;
				ilog.LogInfoLine("This is a new site. Creating site and initial clearing around village.");
			}
			// old site; remove old data
			else
			{
				VillageSites.remove(village_id);
				
				if(vdata.area == 0 && vdata.population == 0){
					activeVillages--;
					ilog.LogInfoLine("Site Removed from active villages count, current active villages: "+activeVillages);
					return;
				}
				
				ilog.LogInfoLine("This is an existing site. Updating site data...");
			}
			
			// set default region for update script
			
			char[] gregionflags = {'a'};
			lastoutput = this.grass.gRegion(REGION_MAP, null, gregionflags, REGION_AGENT_RES, REGION_AGENT_RES, run_quiet);
			ilog.LogInfoLine(lastoutput);
			String vector_site_location2 = initial_year+vlg_id+"site2@"+mapset;
			if ( new_site )
			{
				// create data file
				sitedata = new CreateFile(mapsetpath, data_fname, true);
				String loc = (vdata.site).replaceAll(",","|");
				sitedata.AddLine(vdata.villageID + "|" + loc);
				sitedata.AddLineReturn();
				sitedata.Finalize();
				
				// create vector map from data file
				String[] parameters = {"format=point", "cat=1", "x=2", "y=3"};
				lastoutput = this.grass.vInASCII(sitedata.FullPathName(), site_map, parameters, run_quiet);
				ilog.LogInfoLine(lastoutput);
				
				//convert vector point to raster and then back again, used to make sure that the village catchment matches the other grids
				String raster_village = "village_temp_locR"+vdata.villageID;
				String use = "cat";
				String type = "point";
				//lastoutput = this.grass.vToRast(vector_site_location, use, type, raster_village, null, null, run_quiet);
				lastoutput = this.grass.vToRast(vector_site_location, raster_village, use, type, null, null, run_quiet);
				ilog.LogInfoLine(lastoutput);
							
				boolean overwrite = true;
				char[] flags = {'v'};
				String feature = "point";
				lastoutput = this.grass.rToVect(raster_village, flags, feature, vector_site_location2, run_quiet, overwrite);
				ilog.LogInfoLine(lastoutput);
				
				// create cost surface map
				char[] rwalkflags = {'k'};
				double max_cost = 0;
				int percent_memory = 0;
				int num_segments = 4;
				String walk_coefficient = "0.72,6.0,1.9998,-1.9998";
				double lambda = 0;
				double slope_factor = -0.2125; //site_map_loc
				lastoutput = this.grass.rWalk(rwalkflags, elev_map_loc, fric_map_loc, cost_map, vector_site_location2 , max_cost, percent_memory, num_segments, walk_coefficient, lambda, slope_factor, run_quiet);
				ilog.LogInfoLine(lastoutput);
			}
			
			// create/update village location based upon site and population			
			char[] rcatchflags = null;
			double a = 0.72;
			double b = 6;
			double c = 1.9998;
			double d = -1.9998;
			double lambda = 0;
			double slope_factor = -0.2125;
			int mapval = 40;
			
			lastoutput = this.grass.rCatchmentPy(rcatchflags, cost_map_loc, elev_map_loc, vector_site_location2, a, b, c, d, lambda, slope_factor, village_catchment_map, vdata.area, mapval);	
			ilog.LogInfoLine(lastoutput);
			
			if ( !new_site )
			{
				lastoutput = this.grass.rMapcalc(village_map + "='if(isnull(\"" + village_catchment_map_loc + "\"), null(), " + this.village_clearing_landcover_value + ")'");
				ilog.LogInfoLine(lastoutput);
			}
			
			else
			{
				lastoutput = this.grass.rMapcalc(village_map + "='if(isnull(\"" + village_catchment_map_loc + "\"), null(), " + this.village_clearing_landcover_value + ")'");
				ilog.LogInfoLine(lastoutput);
			}
			
			lastoutput = this.grass.rVillagesPy(landcover_map_loc, village_map_loc, this.village_clearing_landcover_value, temp_landcover_map);
			ilog.LogInfoLine(lastoutput);
			
			//get coordinates of new village location and return to village
			lastoutput = this.grass.gRegion(REGION_MAP, null, gregionflags, REGION_AGENT_RES, REGION_AGENT_RES, run_quiet);
			ilog.LogInfo(lastoutput);
			
			int num_steps = 255;
			String no_value= "0";
			String field_seperator = ",";
			char[] rstatsflags = {'1', 'n', 'g'};
			result = this.grass.rStats(rstatsflags, village_map_loc, null, field_seperator, no_value, num_steps);
	
			if ( result == null )
			{
				throw new NullPointerException("GRASS village site clearing query returned null.");
			}

			// create response message to village
			//	existing message type for this agent
			if ( this.AgentResponses.containsKey(msg_key) ){
				msg = (AgentMsg) this.AgentResponses.remove(msg_key);
			}
			// new agent and/or new message type for this agent
			else{
				msg = new AgentMsg(TypeAgents.VILLAGE, vdata.villageID, Messages.SITE);
			}
			ilog.LogInfo("Site query result: [east, north]");
			
			ArrayList<String> values = new ArrayList  <String> ();
			String value = null;
			String delims = "\n";
			String[] split = result.split(delims);

			for (int p=0; p < split.length-1; p++){
				// Parse result from response.
				// note: output is in the format of [east],[north],[value]. Ignore [value]
				String svalue = split[p].substring(0, split[p].lastIndexOf(","));
				msg.Add(svalue);
				ilog.LogInfo(svalue + "   ");
				// Add result to village data object too
				vdata.AddLocation(svalue);	
			}			
			
			ilog.LogInfoLine("IM: returning " + msg.Size() + " cells for village " + vdata.villageID);
			this.AgentResponses.put(msg_key, msg);
		}
		
		catch (Exception e)
		{
			ilog.LogErrorLine("An error occurred while getting village " + vdata.villageID + " site data.\n" +
					e.getMessage());
		}
	
		finally
		{
			
		}		
					
		// denote update and store village site data in IM
		vdata.updated = true;
		VillageSites.put(village_id, vdata);
		
		ilog.LogInfoLine("Returning from UpdateSite()");
		System.out.println();
		System.out.println("~	Returning from UpdateSite()");
		System.out.println();
				
	}
	
	private void RunLandscapeEvol () throws IOException 
	{
		ilog.LogInfoLine("	IM: RunLandscapeEvol(): ");
		System.out.println();
		System.out.println("~	IM: RunLandscapeEvol(): This takes a while too...");
		System.out.println();
		
		final String elevation_map = CurrentMapLoc(TypeMaps.ELEVATION);
		final String bedrock_map = CurrentMapLoc(TypeMaps.BEDROCK);
		String last_output = null;
		boolean run_quiet = true;
		String soil_K_factor = null;
		if ( TypeMaps.SOILS_KFACTOR != null ){
			soil_K_factor = CurrentMapLoc(TypeMaps.SOILS_KFACTOR);
		}	
		else{	soil_K_factor = String.valueOf(this.soil_kvalue); }
		
		final String C_factor =  cm.NextYear() + TypeMaps.LCVR_CFACTOR.toString() + "@" + mapset;
		char[] flags = {'s'}; // was p and g
		String stats_file = this.GRASSPATH + this.GRASSLOCATION + "/" + this.MAPSET + "/" + mapset + "_"+"Stats";

		try
		{
			char[] gregionflags = {'g', 'p'}; 
			last_output = this.grass.gRegion(CurrentMapLoc(TypeMaps.ELEVATION), null, gregionflags, -1, -1, run_quiet);
			ilog.LogInfoLine(last_output);
			last_output = this.grass.rLandscapeEvolPy(flags, elevation_map, bedrock_map, cm.NextYear(), TypeMaps.ELEVATION.toString(), TypeMaps.SOILDEPTH.toString(), 
					soil_density, this.storm_precip, this.storms, this.storm_speed, this.storm_length, stream_transport, load_exponent, stats_file,
					r_factor, soil_K_factor, C_factor, Double.toString(kappa), smoothing, cutoff1,  cutoff2, cutoff3, run_quiet);	
			ilog.LogInfoLine(last_output);	
			
			if (this.keepRasterMaps){ }else{
			char[] deleteflags = {'f','b'};
			last_output = this.grass.gMRemoveRast("*V*", deleteflags, run_quiet);
			ilog.LogInfoLine(last_output);
			last_output = this.grass.gMRemoveRast("*temp_landcover*", deleteflags, run_quiet);
			ilog.LogInfoLine(last_output);
			last_output = this.grass.gMRemoveRast("*Returns_map*", deleteflags, run_quiet);
			ilog.LogInfoLine(last_output);
			last_output = this.grass.gMRemoveRast("*woodgathering*", deleteflags, run_quiet);
			ilog.LogInfoLine(last_output);
			}
			
		}
		catch (IOException ioe)
		{
			ilog.LogErrorLine("ERROR: IOException encountered during execution of soil dynamic script.");
			ilog.LogErrorLine(ioe.getMessage());
			throw ioe;
		}  
		
		ilog.LogInfoLine("Returning from RunSoilDyamicsModel()");
		System.out.println();
		System.out.println("~	IM: Returning from RunLandscapeEvol()");
		System.out.println();
	}

	private void RunLandAssesser (ArrayList LandChoices ) throws IOException 
	{
		System.out.println();
		System.out.println("~	IM: RunLandAssesser()");
		System.out.println();
		ilog.LogInfoLine("IM: Running RunLandAssesser()");
		final String cfactor = cm.NextYear() + TypeMaps.LCVR_CFACTOR.toString();
		//boolean run_quiet = false;
		String last_output;
		ArrayList LandChoicesList = LandChoices;
		LandRequestString = "";
		String dlm = ";";
		/*
		 * LandRequestMsg = Integer.toString(this.myID )+ dlm + Integer.toString(this.villageID) + dlm + 
		Integer.toString(this.population) + dlm + Double.toString(this.wheat_proportion) + dlm + Double.toString(this.barley_proportion) + 
		dlm + Integer.toString(farmedLandNeed) + dlm+ Integer.toString(grazedLandNeed);
		 */
		for (int p=0; p < LandChoicesList.size(); p++)
		{
				if (p == LandChoicesList.size()-1){
					LandRequestString = LandRequestString + LandChoicesList.get(p);
				}else{
					LandRequestString = LandRequestString + LandChoicesList.get(p) + dlm;
				}
		}

		try{
			if (climateDataSeries){
				this.storms = (int) climateData [0] [cm.IntYear()] ; // storms/yr
				this.r_factor = climateData [1] [cm.IntYear()] ;// r factor
				this.annual_precip = climateData [2] [cm.IntYear()];// annual precip
				this.storm_speed = climateData [3] [cm.IntYear()];// storm speed
				this.storm_length = climateData [4] [cm.IntYear()];// storm length
				this.storm_precip = climateData [5] [cm.IntYear()];//storm precip
			}

			char[] gregionflags = {'a'};
			last_output = this.grass.gRegion(REGION_MAP, null, gregionflags, REGION_AGENT_RES, REGION_AGENT_RES, run_quiet);
			ilog.LogInfo(last_output);	
			
			if (cm.IntYear() > 0 ){
				int vId = 0;
				while (vId < this.totalVillages){ 
					last_output = this.grass.gCopyRast(cm.PrevYear()+"village_"+vId+"_"+ TypeMaps.CLEARING, CurrentMap("village_"+vId+"_", TypeMaps.CLEARING), run_quiet);
					vId++;
				}
			}
						
			//Combine Multiple Cost Surface Maps and Clearing Maps for each village into one variable with multiple maps
			String CostSurfaceMaps = CurrentMap("village_0_", TypeMaps.COSTSFC);
			String villageLand = CurrentMap("village_0_", TypeMaps.CLEARING);
			String VillageMaps = CurrentMap("village_0_", TypeMaps.CLEARING);
			int vId = 0;
			System.out.println("Disregard Map Already Exists Error:");
			if(this.totalVillages > 1){
				vId = 1;
				while (vId < this.totalVillages){
				CostSurfaceMaps = CostSurfaceMaps + ","+CurrentMap("village_" + vId + "_", TypeMaps.COSTSFC);
				VillageMaps = VillageMaps + ","+CurrentMap("village_" + vId + "_", TypeMaps.CLEARING);
				villageLand = cm.Year()+"villageland";
				vId++;	
				}
			}
			last_output = this.grass.rPatch(VillageMaps, villageLand, run_quiet);

			final String next_landcover = cm.NextYear() + TypeMaps.LANDCOVER.toString();
			final String next_fertility = cm.NextYear() + TypeMaps.FERTILITY.toString();
			final String impacts_map = CurrentMap(TypeMaps.IMPACTS);

		String statsfile = this.statsfile_path+this.statsfile_name;
		String fertilstatsfile = this.statsfile_path+this.fertil_stats_name;
		String maxlandcov = null;
		if (this.max_landcover_map != null){
			maxlandcov = this.max_landcover_map;
		}
		else{
			maxlandcov = Double.toString(this.maxlcov);
		}
		System.out.println("Running rLandAssessPy(): This takes a while...");
		System.out.println();
		System.out.println();
		
		double dropval = 40; /// TEMP FIX SINCE THIS VARIABLE IS NOT YET IN USE
		String farmbreaks = Double.toString(this.lcBreakPoint1)+","+Double.toString(this.daBpoint1)+";"+Double.toString(this.lcBreakPoint2)+","+Double.toString(this.daBpoint2);
		char[] flags= {'t'};
		if (!this.landTenure){
			flags = null;
		}		
		
		System.out.println("ASSESS INPUT: "+LandRequestString+" "+this.annual_precip+" "+ this.max_barley_yeld+" "+ this.max_wheat_yeld+" "+ this.wood_pc+" "+ this.ovicaprid_density_factor+" "+ 
				this.degrade_rate+" "+ this.soil_recovery+" "+ this.wood_intensity+" "+ this.wooddistweight+" "+ this.gdistweight+" "+ this.lcovweight+" "+ this.fdistweight+" "+ this.sfertilweight+" "+ 
				this.sdepthweight+" "+ this.maxfarmcost+" "+ this.maxgrazecost+" "+ maxlandcov+" "+ CurrentMap(TypeMaps.SLOPE)+" "+ CurrentMap(TypeMaps.LANDCOVER)+" "+ CurrentMap(TypeMaps.FERTILITY)+" "+
				CurrentMap(TypeMaps.SOILDEPTH)+" "+ villageLand+" "+ CostSurfaceMaps+" "+ cm.Year()+" "+ next_landcover+" "+ next_fertility+" "+impacts_map+" "+ this.FERTILIY_COLOR_PATH+" "+
				this.LANDCOVER_RECLASS_PATH+" "+ this.LANDCOVER_COLOR_PATH+" "+ this.USER_PATH + "Scripts" + File.separator+" "+ statsfile+" "+fertilstatsfile+" "+farmbreaks+" "+dropval+" "+cm.PrevYear()+" "+flags);
		
		ilog.LogInfo("ASSESS INPUT: "+LandRequestString+" "+this.annual_precip+" "+ this.max_barley_yeld+" "+ this.max_wheat_yeld+" "+ this.wood_pc+" "+ this.ovicaprid_density_factor+" "+ 
				this.degrade_rate+" "+ this.soil_recovery+" "+ this.wood_intensity+" "+ this.wooddistweight+" "+ this.gdistweight+" "+ this.lcovweight+" "+ this.fdistweight+" "+ this.sfertilweight+" "+ 
				this.sdepthweight+" "+ this.maxfarmcost+" "+ this.maxgrazecost+" "+ maxlandcov+" "+ CurrentMap(TypeMaps.SLOPE)+" "+ CurrentMap(TypeMaps.LANDCOVER)+" "+ CurrentMap(TypeMaps.FERTILITY)+" "+
				CurrentMap(TypeMaps.SOILDEPTH)+" "+ villageLand+" "+ CostSurfaceMaps+" "+ cm.Year()+" "+ next_landcover+" "+ next_fertility+" "+impacts_map+" "+ this.FERTILIY_COLOR_PATH+" "+
				this.LANDCOVER_RECLASS_PATH+" "+ this.LANDCOVER_COLOR_PATH+" "+ this.USER_PATH + "Scripts" + File.separator+" "+ statsfile+" "+fertilstatsfile+" "+farmbreaks+" "+dropval+" "+cm.PrevYear()+" "+flags);
					
			String assess_output = this.grass.rLandAssessPy(LandRequestString, this.annual_precip, this.max_barley_yeld, this.max_wheat_yeld, this.wood_pc, this.ovicaprid_density_factor, 
					this.degrade_rate, this.soil_recovery, this.wood_intensity, this.wooddistweight, this.gdistweight, this.lcovweight, this.fdistweight, this.sfertilweight, 
					this.sdepthweight, this.maxfarmcost, this.maxgrazecost, maxlandcov, CurrentMap(TypeMaps.SLOPE), CurrentMap(TypeMaps.LANDCOVER), CurrentMap(TypeMaps.FERTILITY),
					CurrentMap(TypeMaps.SOILDEPTH), villageLand, CostSurfaceMaps, cm.Year(), next_landcover, next_fertility, impacts_map, this.FERTILIY_COLOR_PATH,
					this.LANDCOVER_RECLASS_PATH, this.LANDCOVER_COLOR_PATH, statsfile,fertilstatsfile, farmbreaks, dropval, cm.PrevYear(), flags);
		
			ilog.LogInfoLine("" );
			ilog.LogInfo("ASSESS OUTPUT: "+assess_output);
	
			/*	
		
		String assess_output = this.grass.rLandAssessPy(LandRequestString, this.annual_precip, this.max_barley_yeld, this.max_wheat_yeld, this.wood_pc, this.ovicaprid_density_factor, 
				this.degrade_rate, this.soil_recovery, this.wood_intensity, this.wooddistweight, this.gdistweight, this.lcovweight, this.fdistweight, this.sfertilweight, 
				this.sdepthweight, this.maxfarmcost, this.maxgrazecost, maxlandcov, CurrentMap(TypeMaps.SLOPE), CurrentMap(TypeMaps.LANDCOVER), CurrentMap(TypeMaps.FERTILITY),
				CurrentMap(TypeMaps.SOILDEPTH), villageLand, CostSurfaceMaps, cm.Year(), next_landcover, next_fertility, impacts_map, this.FERTILIY_COLOR_PATH,
				this.LANDCOVER_RECLASS_PATH, this.LANDCOVER_COLOR_PATH, this.USER_PATH + "Scripts" + File.separator, statsfile,fertilstatsfile);
	*/
		
		System.out.println("ASSESS OUTPUT: "+assess_output);


		
		
//output ;HH#:avg_agri_return,avgoc_return,wheat_yield,barley_yield,oc_yield;HH#:avg_agri_return,avgoc_return,wheat_yield,barley_yield,oc_yield;HH#:avg_agri_return,avgoc_return,wheat_yield,barley_yield,oc_yield
			last_output = this.grass.rCFactorPy(next_landcover, cfactor, CFACTOR_RECLASS_PATH, CFACTOR_COLOR_PATH);
			ilog.LogInfoLine(last_output);

			ArrayList RequestingAgents = new ArrayList();
			Pair agent = new Pair();
			TypeAgents agentType = TypeAgents.HOUSEHOLD;
			int agentID;
 	        String data = "";
 	        String[] sdata = null;
 	        String[] mdata = null;
 	        String dlm_regex = ";";
 	        data = assess_output;
 	     //Split each HH, first value is useless
 	        sdata = data.split(dlm_regex); 
 	        dlm_regex = ":";
			for ( int i = 1; i < sdata.length; i++ )
			{
				mdata = sdata[i].split(dlm_regex);
				agent.key = agentType;
				agent.value = Integer.parseInt(mdata[0]);		
				RequestingAgents.add(agent);
				SendDataToAgents(RequestingAgents, mdata[1], Messages.HARVEST);
			
			}			

		}	
		
		catch (IOException ioe)
		{
			System.out.println("ERROR: IOException encountered during execution of RunLandAsseser().");
			ilog.LogErrorLine("ERROR: IOException encountered during execution of RunLandAsseser().");
			ilog.LogErrorLine(ioe.getMessage());
			throw ioe;
		}  
		System.out.println();
		System.out.println("~	Returning From: RunLandAssesser()");
		System.out.println();
		ilog.LogInfoLine("	Returning From:  RunLandAssesser()");
		
	}

	private void SendDataToAgents ( ArrayList AgentsList, Object data, Messages msg_type )
	{		
		AgentMsg msg = null;
		Pair agent = null;
		TypeAgents agent_type = null;
		int agent_id = -1;
		String msg_key = null;
		
		//ilog.LogInfoLine("IM: SendDataToAgents()...");
		// build messages to each agent that requested a value from this cell
		Iterator agents = AgentsList.listIterator();
		
		if ( agents.hasNext() )
			//ilog.LogInfo("The following agents will be sent the data: ");
		
		while ( agents.hasNext() )
		{
			agent = (Pair) agents.next();
			agent_type = (TypeAgents) agent.key;
			agent_id = ((Integer) agent.value).intValue();
			msg_key = agent_type.toString() + agent_id + msg_type.toString();
			
			// check if messages already exist to this agent
			// msg_key is unique for each agent and message type
			if ( this.AgentResponses.containsKey(msg_key) ){
				msg = (AgentMsg) this.AgentResponses.remove(msg_key);
			}
			// new agent and/or new request type for that agent
			else{
				msg = new AgentMsg(agent_type, agent_id, msg_type);
			}
			// add new data and store in temp hashtables
			if ( data.getClass().getName() == ArrayList.class.getName() ){
				msg.Add((ArrayList) data);
			}
			else{
				msg.Add(data);
			}
			//ilog.LogInfo(agent_type.toString() + " " + agent_id + "   ");
			this.AgentResponses.put(msg_key, msg);
		}
		ilog.LogInfoLine("Returning from SendDataToAgents()");	
	}
	
	// return current map name
	private String CurrentMap ( TypeMaps map )
	{
		String full_name = "";
		
		if ( map.IsDynamic() ){
			full_name += cm.Year();
		}
		if ( map.toString().compareTo("") != 0 ){
			full_name += map.toString();
		}
		else{
			full_name = "";
		}
		return full_name;
		
	}
	
	// return current map name with mapset suffix
	private String CurrentMapLoc ( TypeMaps map )
	{
		String full_name = CurrentMap(map);
		
		if ( full_name.compareTo("") != 0 ){
			return (full_name + "@" + mapset);
		}
		else{
			return "";
		}
	}
	
	// return current map name
	private String CurrentMap ( String specifier, TypeMaps map )
	{
		String full_name = "";
		
		if ( map.IsDynamic() ){
			full_name += cm.Year();
		}
		if ( specifier != null ){
			full_name += specifier;
		}
		full_name += map.toString();
		
		return full_name;
		
	}
	
	// return current map name with mapset suffix
	private String CurrentMapLoc ( String specifier, TypeMaps map )
	{
		String full_name = CurrentMap(specifier, map);
		
		return (full_name + "@" + mapset);
	}
	
	// a collection of the maps required to provide cell data to the household agent
	// call every cycle to update name(s) with proper prefix
	private void SetCellDataMaps ()
	{
		ilog.LogInfoLine("Setting maps required for household land value determination.");

		this.CellDataMaps = new TypeMaps[4];
		this.CellDataMaps[0] = TypeMaps.SOILDEPTH;	// soil depth map
		this.CellDataMaps[1] = TypeMaps.LANDCOVER;	// land cover map 
		this.CellDataMaps[2] = TypeMaps.SLOPE;
		this.CellDataMaps[3] = TypeMaps.FERTILITY;
	}
	
	// Returns a comma delimited string of the map names using the current calendar prefixes
	public String GetCellDataMaps ()
	{
		String maps = "";
		int num_maps = this.CellDataMaps.length;
		
		for ( int m = 0; m < num_maps; m++ )
		{
			maps += CurrentMap(this.CellDataMaps[m]);
			
			if ( m < num_maps - 1 ){			// add a comma if not the last map
				maps += ",";
			}
		}
		
		return maps;
		
	}
	
	////////////////////////////////////////
	//      SimView Helper Functions      //
	////////////////////////////////////////
	public String getTooltipText()
	{
		return
			super.getTooltipText()
			+ "\n" + "year: " + cm.Year()
			+ "\n" + "village sites: " + VillageSites.size();
	}
	
}
