/*
 * MVillage.java
 *
 * Package Version: 4.0
 *   Class Version: 1.0
 *   
 * 		   Project: MEDLAND Project
 * 					Arizona State University
 * 
 * 			Author: Gary R. Mayer
 *   		  Date: 04 March 2008
 * 
 * This file contains the MVillage coupled model. This model represents a collection of
 * households. 
 * 
 * Upon object construction, a MVillageManager, a MHouseholdDemultiplexer, and all of the
 * Households models initially a part of the MVillage, are created.
 * 
 * Class Fixes / Additions:
 * 1.0:
 * 	- Uses the ModelInitialization form and InitParameters object to set model
 * 		parameters.
 * 
 * Class Bugs / Issues:
 * 1.0:
 *  - None.
 */

package PenaguilaALv4;

import java.awt.*;
import java.io.File;
import java.util.ArrayList;
import GenCol.Pair;
import view.modeling.*;
import view.*;



public class MVillage extends ViewableDigraph
{
	private static int ticket = -1;		// village ID generator.
	private int myID;					// id unique to each village
	private MVillageManager vm;			// manages household dynamic coupling.
	private MHouseholdDemultiplexer dm;	// demultiplexes household messages.
	private ArrayList HouseholdIDs;		// IDs of households belonging to this village
//	private VillageTransducer vt;		// captures data about the households
	public String log_path = null;
	
	public MVillage ()
	{
		this(-1, null, new Pair(new Integer(3), new Integer(3)), 3, 6, 250, null);
		
	}
	
	public MVillage ( Pair loc, int numHouseholds, int peoplePerHousehold, int hhMaxDistanceCost, InitParameters ip)
	{
		this(-1, null, loc, numHouseholds, peoplePerHousehold, hhMaxDistanceCost, ip);
		
	}
	
	public MVillage ( int id, String name, Pair loc,
			int numHouseholds, int peoplePerHousehold, int hhMaxDistanceCost,
			InitParameters ip)
	{	
		super( (name != null) ? name : "Village" );
		myID = (id == -1) ? ++ticket : id;
		HouseholdIDs = new ArrayList();
		
		String mapsetPath = ip.GISDBASE + ip.location  + File.separator;
		String mapsetName = ip.mapset;
		File dir = new File(mapsetPath + mapsetName);
		int run_num = 1;
		String new_mapset = null;
		if( dir.exists()){
			String run = Integer.toString(run_num);	    
			new_mapset = mapsetName + "_Run" + run;
			dir = new File(mapsetPath + new_mapset);	
			while ( dir.exists() )
			{
				run_num++;
				run = Integer.toString(run_num);
				new_mapset = mapsetName + "_Run" + run;
				dir = new File(mapsetPath + new_mapset);            
			}
		}
		
	    this.log_path = mapsetPath + new_mapset + File.separator + "APSimLogs" + File.separator;
	    //System.out.println("finallog: "+ this.log_path);
		//this.log_path = ip.APSIMPath + "/Temp/";
		
		addInport("in");
		addOutport("land");
		addOutport("ext_msgs");
		
		dm = new MHouseholdDemultiplexer("HhMsgPasser");
		addCoupling(this, "in", dm, "msg_in");
		add(dm);

		boolean village_log = ip.VillageLogOn; 
		
		vm = new MVillageManager("VManager", this, loc, this.log_path, village_log);
		addCoupling(dm, "vmanager", vm, "requests");
		addCoupling(vm, "land", this, "land");
		add(vm);
			
		// create initial households and add to village.
		for ( int i = 0; i < numHouseholds; i++ )
		{
			MHousehold hh = new MHousehold(peoplePerHousehold, this.myID, ip);
			Join(hh);
		}
		
		initialize();
	}
	
	public int GetID()
	{
		return myID;
	}
	
	public void Join ( final MHousehold hh )
	{
		// add household to village collection.
		addCoupling(hh, "land", this, "land");
		add(hh);
		
		Integer hhID = hh.GetID();
		HouseholdIDs.add(hhID);
		String new_port = "hh_" + hhID.toString();
		
		// couple household and village manager / transducer.
		addCoupling(hh, "vmanager", vm, "requests");
		vm.addOutport(new_port);
		addCoupling(vm, new_port, hh, "info");
		
		// couple household to village manager and and village messenger.
		dm.addOutport(new_port);
		addCoupling(dm, new_port, hh, "info");
		dm.AddHousehold(hhID, new_port);
		
	}
	
	public void Leave ( final MHousehold hh )
	{
		// remove household from village collection.
		removeCoupling(hh, "land", this, "land");
		
		remove(hh);
		
		// decouple household and village manager / transducer.
		removeCoupling(hh, "vmanager", vm, "requests");
//		removeCoupling(hh, "council", vt, "pop");
		
		// decouple household and village messenger.
		Integer hhID = hh.GetID();
		String hh_port = "hh_" + hhID;
		dm.RemoveHousehold(hhID);
		removeCoupling(dm, hh_port, hh, "info");
		dm.removeOutport(hh_port);
		
	}
	
	public ArrayList GetHouseholdMembers ()
	{
		return HouseholdIDs;
		
	}

    /**
     * Automatically generated by the SimView program.
     * Do not edit this manually, as such changes will get overwritten.
     */
    public void layoutForSimView()
    {
        preferredSize = new Dimension(599, 241);
        ((ViewableComponent)withName("    Household_2    ")).setPreferredLocation(new Point(224, 168));
        ((ViewableComponent)withName("    Household_0    ")).setPreferredLocation(new Point(222, 24));
        ((ViewableComponent)withName("HhMsgPasser")).setPreferredLocation(new Point(-1, 126));
        ((ViewableComponent)withName("   VManager   ")).setPreferredLocation(new Point(-7, 47));
        ((ViewableComponent)withName("    Household_1    ")).setPreferredLocation(new Point(222, 97));
    }
}
