/*
 * MVillageDemultiplexer.java
 * 
 * Package Version: 2.0
 *   Class Version: 1.1
 *  
 * 		   Project: MEDLAND Project
 * 					Arizona State University
 * 
 * 			Author: Gary R. Mayer
 *   		  Date: 28 January 2008
 * 
 * This file contains the MSettlements coupled model. It represents the
 * entire scenrio in which the human agent models act. This includes both
 * the agents themselves and the abstracted landscape model data upon which
 * the agents make decisions.
 * 
 * Upon object construction, the landscape and villages coupled models are created.
 * 
 * Class Fixes / Additions:
 * 1.1:
 * 	- Added multi-village management and household-to-village cross-referencing.
 * 1.0:
 *  - Published version.
 * 0.2:
 * 	- Simulation simplified to only encompass 1 village.
 * 0.1:
 * 	- Messages input from landscape and sent to appropriate villages, to be demultiplexed
 * 		by HouseholdDemultiplexer for delivery to specific household.
 * 
 * Class Bugs / Issues:
 * 1.1:
 * 	- None.
 * 1.0:
 *  - Implement actual multi-village functionality and household-to-village cross-referencing,
 *  	which will enable a message to be addressed to a household to be looked up and determine
 *  	to which village the household currently belongs.
 * 0.2:
 * 	- Currently, all messages go to Village_0 with the expectation that there is only 1 village.
 * 		Revise for multiple villages.
 * 0.1:
 * 	- Current scheme requires specifying village ID in message header. This creates too
 * 		tight of a coupling as models (household and cell) sending messages to a household
 * 		must have both the household ID and the current village ID. Recommend modifying schema
 * 		to include only household ID. Local demultipelxers keep track of local households and
 * 		send messages if household is known. If unknown, message is passed to MVillageDemultiplexer
 * 		(this model) which maintains a global cross-reference of households and villages.
 * 
 */
package PenaguilaALv4;

import java.util.*;

import model.modeling.message;
import GenCol.Queue;
import view.modeling.*;


public class MVillageDemultiplexer extends ViewableAtomic
{
	private Hashtable Villages;
	private Hashtable Households;
	private Queue MsgQueue;
	private message m;
	
	public MVillageDemultiplexer ()
	{
		this("VMsgPasser");
		
	}
	
	public MVillageDemultiplexer ( String name )
	{
		super(name);
		Villages = new Hashtable();
		Households = new Hashtable();
		
		addInport("msgs");
		// village output ports added by MSettlements model
		
	}
	
	public void initialize ()
	{
		MsgQueue = new Queue();
		
		passivate();
		
	}
	
	public void deltext ( double e, message x )
	{
		Continue(e);

		for (int i = 0; i < x.getLength(); i++)
		{
			if ( messageOnPort(x, "msgs", i) )
			{
				AgentMsg msg = (AgentMsg) x.getValOnPort("msgs", i);
				MsgQueue.addLast((AgentMsg) msg);
			}
		}
		
		if ( !MsgQueue.isEmpty() )
			holdIn("send_msgs", 0);
		
		else
			passivate();
		
	}
	
	public void deltint ()
	{	
		while ( !MsgQueue.isEmpty() )
			holdIn("send_msgs", 0);
		
		passivate();
		
	}
	
	// Manage external events first.
    public void deltcon ( double e, message x )
    {
   		deltext(0, x);
   		deltint();
   		
    }
    
	public message out ()
	{
		AgentMsg msg;
		String port;
		
		m = new message();

		while ( !MsgQueue.isEmpty() )
		{
			msg = (AgentMsg) MsgQueue.removeFirst();
			
			// broadcast to all villages
			if ( msg.agentID == -1 )
			{
				Enumeration vlg_port = Villages.elements();
			
				while ( vlg_port.hasMoreElements() )
				{
					port = (String) vlg_port.nextElement();
					m.add(makeContent(port, msg));
				}
			}
				
			// send to specific village
			else
			{
				port = GetPortFromID(msg.agentType, msg.agentID);
				
				// Send to agent in this village.
				if ( port != null )
					m.add(makeContent(port, msg));
			}
		}
		
		return m;
/*		
		AgentMsg msg;
		String port;
		
		m = new message();

		while ( !MsgQueue.isEmpty() )
		{
			msg = (AgentMsg) MsgQueue.removeFirst();
			
			if ( msg.agentType == TypeAgents.VILLAGE )
				port = "v_0";				// send all household messages to village_0 for now
//				port = (String) Villages.get(new Integer(msg.agentID));
			
			// send household message to village where household is currently located
			else
				port = "v_0";				// send all household messages to village_0 for now
//				port = (String) Households.get(new Integer(msg.agentID));
			
			m.add(makeContent(port, msg));
		}

	    return m;
*/	    
	    
	    
	}
	
	public String getTooltipText ()
	{
		return
			super.getTooltipText()
			+ "\n Number of queued messages: " + MsgQueue.size()
			+ "\n Number of villages: " + Villages.size();
		
	}
	
	public void AddVillage ( Integer ID, String port )
	{
		Villages.put((Integer) ID, (String) port);
		
	}
	
	public void AssignHousehold ( Integer hhID, Integer vlgID )
	{
		if ( Households.containsKey(hhID) )
			Households.remove(hhID);
		
		Households.put(hhID, vlgID);
		
	}
	
	public void AssignHouseholds ( ArrayList hhIDs, Integer vlgID )
	{
		Iterator hh = hhIDs.iterator();
		
		while ( hh.hasNext() )
		{
			Integer hhID = (Integer) hh.next();
			
			if ( Households.containsKey(hhID) )
				Households.remove(hhID);
			
			Households.put(hhID, vlgID);
		}
		
	}
	
	private String GetPortFromID ( TypeAgents agent_type, int agent_id )
	{
		String port = null;
		Integer id = new Integer(agent_id);
		
		if ( (agent_type == TypeAgents.VILLAGE)  && Villages.containsKey(id) )
			port = (String) Villages.get(id);
		
		else if ( (agent_type == TypeAgents.HOUSEHOLD) && Households.containsKey(id) )
			port = "v_" + ((Integer) Households.get(id)).toString();
		
		return port;
		
	}
	
}
