/*
 * MVillageManager.java
 * 
 * Package Version: 2.0
 *   Class Version: 1.1
 * 
 * 		   Project: MEDLAND Project
 * 					Arizona State University
 * 
 * 			Author: Gary R. Mayer & Sean Bergin
 * 
 *   		  Date: 28 June 2007
 *     Last Update: July, 2010
 * 
 * This file contains the MVillageManager atmoic model. This model maintains the attributes
 * that define a village (population, location, etc.) and provides functionality to
 * dynamically couple households in this village. 
 * 
 * Class Fixes / Additions:
 * 1.1:
 *  - 
 * 1.0:
 *  - Published version.
 * 0.2:
 * 	- Model household data management revised to use hashtable.
 * 	- VM now receives initial household population data.
 * 0.1:
 * 	- Model created. Test inputs show model works correctly.
 * 
 * Class Bugs / Issues:
 * 1.0:
 *  - When household sends release message, VillageManager inappropriate handles resulting in
 *  	null value and exception error. ManagedLand stored as (agent_type, agent_id).
 * 0.1:
 * 	- Though the model object is created, it does not receive any updates from the households. Thus,
 * 		it has no knowledge of village population or even the number of households.
 * 	- Devise approach to increase/decrease residence as village population grows. Use addLast and
 * 		removeLast, respectively.
 * 	- Provide incoming households with primary residence cell (the first in the queue)
 */

package PenaguilaALv4;


import java.util.*;
import view.modeling.ViewableAtomic;
import GenCol.Pair;
import GenCol.Queue;
import model.modeling.message;


public class MVillageManager extends ViewableAtomic
{
	private MVillage myVillage;		// MVillage that this object helps manage
	private Pair location;			// village location cell coordinates
	private Queue Residence;		// cells used as residence for this village
	private int population;			// total village population
	private Hashtable Households;	// household populations cross-referenced by household ID
	private Hashtable ManagedLand;	// table of land around the village managed by the village or its households
	private Queue Requests;			// message requests from households
	private Hashtable Responses;	// message responses to households
	private AgentMsg loc_msg;		// msg to update village location / size
	private boolean new_pop;		// flag indicating if population was updated
	public boolean output = false;
	private boolean initializedLogs = false;
	public String log_path = null;
	
	InfoLog ilog;
	
	public MVillageManager()
	{
		this("Manager", new MVillage(), new Pair(new Integer(0), new Integer(0)),  "error", false);
	}
	
	public MVillageManager ( String nm, MVillage v, Pair loc, String log, Boolean outputlog )
	{
		super("   " + nm + "   ");
		
		myVillage = v;
		location = loc;
		this.log_path = log;
		this.output = outputlog;

		addInport("requests");
		addOutport("land");
		
	}
	
	public void initialize ()
	{
		Residence = new Queue();
		Residence.addFirst((Pair)location);
		population = 0;
		Households = new Hashtable();
		ManagedLand = new Hashtable(); //t
		Requests = new Queue();
		Responses = new Hashtable();
		new_pop = false;
		loc_msg = null;
		ilog = new InfoLog();

		holdIn("await_pop", INFINITY);	// wait for households to provide population
	}
	
	public void deltext ( double e, message x )
	{
		if (!initializedLogs){
			////////////<<   LOG SETTINGS   >> /////////////////
			output = false;
			if (output){
			String[] data_types = { "population" };
			ilog.SetFileInfo(this.log_path, "Village_Manager" + this.myVillage.GetID() + "_log.txt", data_types);
			ilog.SetDisplay(InfoLog.ERRORS);
			ilog.StartLog(InfoLog.DATA, true);
			//ilog.StartLog(InfoLog.INFO, true);
			ilog.LogInfoLine("Village " + this.myVillage.GetID());
			}
			else{
				ilog.SetDisplay(InfoLog.ERRORS);
			}
			
			initializedLogs = true;
		}
		
		
		Continue(e);
		AgentMsg msg = null;
		Queue LandRequests = null;
		Queue GrazingRequests = null;

		try
		{
			for (int i = 0; i < x.getLength(); i++)
			{
				if ( messageOnPort(x, "requests", i) )
				{
					msg = (AgentMsg) x.getValOnPort("requests", i);
					
					// handle release messages first
					if ( msg.msgType == Messages.RELEASE )
						Requests.addFirst((AgentMsg)msg);
					
					else
						Requests.addLast((AgentMsg)msg);
				}
			}
			
			while ( !Requests.isEmpty() )
			{
				msg = (AgentMsg) Requests.removeFirst();
				Messages request = msg.msgType;
				int agent_id = msg.agentID;
				TypeAgents agent_type = msg.agentType;
				
				if ( request == Messages.SITE )
				{
					UpdateVillageSite(msg);
				}
				
				if ( request == Messages.POPULATION )
				{
					Integer hhID = new Integer(msg.agentID);
					Integer hhPop = (Integer) msg.Next();
	
					UpdatePopulation(hhID, hhPop);
					new_pop = true;
				}
				
				else if ( request == Messages.RELEASE )
				{
					String landID = (String) msg.Next();
					while ( landID != null )
					{
						Pair manager = (Pair) ManagedLand.get(landID); //t
						TypeAgents mngr_type = (TypeAgents) manager.key;
						int mngr_id = ((Integer) manager.value).intValue();
						
						if ( (mngr_type == agent_type) && (mngr_id == agent_id) )
							ManagedLand.remove(landID);//t
						
						landID = (String) msg.Next();
					}
				}
				
				else if ( request == Messages.PLAN )
				{
					if ( LandRequests == null )
						LandRequests = new Queue();
					
					Integer hhID = new Integer(msg.agentID);
					
					String land = (String) msg.Next();
					while ( land != null )
					{
						// add all land to LandRequest Queue as (requestor, coord) pairs
						Pair land_rqst = new Pair(hhID, land);
						LandRequests.add((Pair)land_rqst);
						land = (String) msg.Next();
					}
				}
							
				else if ( request == Messages.GRAZINGPLAN )
				{
					if ( GrazingRequests == null )
						GrazingRequests = new Queue();
					
					Integer hhID = new Integer(msg.agentID);
					
					String land = (String) msg.Next();
					while ( land != null )
					{
						// add all land to LandRequest Queue as (requestor, coord) pairs
						Pair land_rqst = new Pair(hhID, land);
						GrazingRequests.add((Pair)land_rqst);
						land = (String) msg.Next();
					}
				}
			}
			
			ilog.LogDataLine("population", " Village "+ this.myVillage.GetID() + ",pop=,"+population);
			
			if ( LandRequests != null )
				ManageLandRqst(LandRequests);
			
			if ( GrazingRequests != null )
				ManageGrazingLandRqst(GrazingRequests);
			
			if ( new_pop )
			{
				// create village footprint; store in managed
				// if a household was managing the land, inform it that it no longer is
				loc_msg = new AgentMsg(TypeAgents.VILLAGE, this.myVillage.GetID(), Messages.SITE);
				int id = myVillage.GetID();
				double density = 0.0159;	 // people per sq.m
				int area = (int) Math.round(population/density);
				String loc = new String(((Integer) location.key).toString() + "," + ((Integer) location.value).toString());
				VillageData vdata = new VillageData(id, loc, population, area);
				loc_msg.Add(vdata);
				new_pop = false;
				
				ilog.LogInfoLine("total_pop:" + population + ",area:" + area);
				holdIn("send_msg", 0.0);
			}
			
			else if ( !Responses.isEmpty() )
				holdIn("send_msg", 0.0);
			
			else
				passivate();
		}
		
		catch (Exception exc)
		{
			ilog.LogErrorLine("Village input exception.\n" + exc.getMessage());
			exc.printStackTrace();
		}
	}
	
	public void deltint ()
	{
		if ( loc_msg != null )
			holdIn("send_msg", 0.0);
		
		else if ( !Responses.isEmpty() )
			holdIn("send_msg", 0.0);
		
		else
			passivate();
	}

	public message out ()
	{
		message m = null;
		String msg_key = null;
		AgentMsg msg = null;
		
		if ( this.loc_msg != null )
		{
			m = new message();
			m.add(makeContent("land", this.loc_msg));
			
			this.loc_msg = null;
		}
		
		if ( !this.Responses.isEmpty() )
		{
			Enumeration agent = null;
			
			if ( m == null )
				m = new message();
			
			agent = this.Responses.keys();
			
			while ( agent.hasMoreElements() )
			{
				msg_key = (String) agent.nextElement();
				msg = (AgentMsg) this.Responses.remove(msg_key);
				
				if ( msg.agentType == TypeAgents.HOUSEHOLD )
					m.add(makeContent("hh_" + msg.agentID, msg));
			}
		}
		
		return m;
	}
	
	public String getTooltipText()
	{
		return
			super.getTooltipText()
			+ "\n Location: " + location.key.toString() + ", " + location.value.toString()
			+ "\n Households: " + Households.size()
			+ "\n Population: " + population
			+ "\n Managed land cells: " + ManagedLand.size()
			+ "\n Number of queued requests: " + Requests.size();
	}
	
	private void UpdateVillageSite ( AgentMsg msg )
	{
		//ilog.LogInfoLine("VillageManager: deltext(): UpdateVillageSite()...");

		// store new village site in a temporary hashtable
		Hashtable TempSites = new Hashtable();
		String new_site = (String) msg.Next();
		
		if ( new_site == null )
			throw new IllegalArgumentException("New site message from IM contains no data.");
		
		//ilog.LogInfo("New site: ");
		
		while ( new_site != null )
		{
			TempSites.put((String)new_site, (Pair)new Pair(TypeAgents.VILLAGE, new Integer(myVillage.GetID())));
			//ilog.LogInfo(new_site + " ");
			new_site = (String) msg.Next();
		}
		
		// search through the existing ManagedLand for sites where the households
		//	are managing the land. if the land is now managed by the village,
		//	notify the household.
		// Note: this also allows land to be released if the village size shrinks
		//	thus allowing households to manage closer land once used by the village.
		
		String curr_site = null;
		Pair agent = null;
		TypeAgents agent_type = null;
		Integer agent_id = null;
		Enumeration site = ManagedLand.keys();
		
		ilog.LogDataLine("village_ts", "tempsites,"+TempSites.toString());
		while ( site.hasMoreElements() )
		{
			curr_site = (String) site.nextElement();
			agent = (Pair) ManagedLand.remove(curr_site);
			agent_type = (TypeAgents) agent.key;
			agent_id = (Integer) agent.value;
			ilog.LogDataLine("village_ts", "curr_site,"+ curr_site);  
			
			
			if ( agent_type == TypeAgents.HOUSEHOLD )
			{
				if ( TempSites.containsKey(curr_site) )
				{
					ilog.LogDataLine("village_ts", "v_hh_match curr_site,"+ curr_site);  
					// send message to household that it no longer manages the land					
 					AgentMsg unmng_msg = null;
					String msg_key = agent_type + agent_id.toString() + Messages.UNMANAGE;

 					if ( this.Responses.containsKey(msg_key) ){
 						unmng_msg = (AgentMsg) this.Responses.remove(msg_key);
 					}
					else	{	unmng_msg = new AgentMsg(TypeAgents.HOUSEHOLD, agent_id.intValue(), Messages.UNMANAGE);}
					
					unmng_msg.Add(curr_site);
					this.Responses.put((String)msg_key, (AgentMsg)unmng_msg);
					
					//ilog.LogInfoLine("New cell site, " + curr_site + " is currently managed by a household" + agent_id.intValue() + ". Sending " + Messages.UNMANAGE.toString() + " message to household");
				}
				
				else
				{
					// add managed land to temporary table
					TempSites.put((String)curr_site, (Pair)agent);
					//ilog.LogInfoLine("Adding " + curr_site);
				}
			}
			
			// managed by village; ignore to release land no longer village managed
		}
		
		// set managed land to new village site and household lands
		ManagedLand = TempSites;
		
		//ilog.LogInfoLine("Returning from UpdateVillageSite()");
		
	}
	
	private void UpdatePopulation ( Integer id, Integer pop )
	{
		// delete old population if not a new household.
		if ( Households.containsKey(id) )
			population -= ((Integer) Households.remove(id)).intValue();

		// insert population and update total.
		Households.put((Integer)id, (Integer)pop);
		population += pop.intValue();
		
		//ilog.LogDataLine("", "Hh:" + id.intValue() + ",Hh_pop:" + pop.intValue());
		//ilog.LogInfoLine("VM: UpdatePopulation(): new population is " + population);
		
	}
	
	private void ManageLandRqst ( Queue LandRequests )
	{
		// randomize requests for fair chance
		Random rand = new Random();
		int numRqsts = LandRequests.size();
		Object obj = null;
		
		//ilog.LogInfo("VM: ManageLandRqst(): randomizing requests...");
		
		for ( int r = 0; r < numRqsts; r++ )
		{
			obj = LandRequests.remove(r);
			LandRequests.add(rand.nextInt(numRqsts), (Object)obj);
		}
		for ( int r = 0; r < numRqsts; r++ )
		{
			obj = LandRequests.remove(r);
			LandRequests.add(rand.nextInt(numRqsts), (Object)obj);
		}

		// search through land requests and assign land
		// provide negative responses as necessary
		Pair land_rqst = null;
		Integer hhID;
		String land;
		Pair requester;
		AgentMsg msg;
		
		while ( !LandRequests.isEmpty() )
		{
			land_rqst = (Pair) LandRequests.removeFirst();
			hhID = (Integer) land_rqst.key;
			land = (String) land_rqst.value;
			String msg_key = TypeAgents.HOUSEHOLD.toString() + hhID.intValue() + Messages.PLAN.toString();
			String result = null;
			
			
			if ( !ManagedLand.containsKey(land) )
			{
				// assign land to this agent and send positive response
				requester = new Pair(TypeAgents.HOUSEHOLD, hhID);
				ManagedLand.put((String)land, (Pair)requester);
				result = "cultivated";
			}
			
			else
			{
				// check if the agent already manages the land
				// don't allow the agent to cultivate village-managed land or land managed by a different household
				requester = (Pair) ManagedLand.get(land);
				TypeAgents agent_type = (TypeAgents) requester.key;
				Integer agent_id = (Integer) requester.value;
				
				if ( (agent_type == TypeAgents.HOUSEHOLD) && (agent_id.intValue() == hhID.intValue()) )
					result = "cultivated";		// same household, send positive response
			
				else
					result = "managed";			// already managed, send negative response
			}
			
			//ilog.LogInfoLine("Request to farm from Hh " + hhID + " for " + land + ". Response: " + result);
			
			// create/append message to agent
			if ( this.Responses.containsKey(msg_key) )
				msg = (AgentMsg) this.Responses.remove(msg_key);
			
			else
				msg = new AgentMsg(TypeAgents.HOUSEHOLD, hhID.intValue(), Messages.PLAN);
			
			msg.Add(new Pair(land, result));			
			this.Responses.put((String)msg_key, (AgentMsg)msg);
		}
		//ilog.LogInfoLine("Returning from ManageLandRqst()");
		
	}
	
	private void ManageGrazingLandRqst ( Queue LandRequests )
	{
		// randomize requests for fair chance
		Random rand = new Random();
		int numRqsts = LandRequests.size();
		Object obj = null;
		
		//ilog.LogInfo("VM: ManageLandRqst(): randomizing requests...");
		
		for ( int r = 0; r < numRqsts; r++ )
		{
			obj = LandRequests.remove(r);
			LandRequests.add(rand.nextInt(numRqsts), (Object)obj);
		}
		for ( int r = 0; r < numRqsts; r++ )
		{
			obj = LandRequests.remove(r);
			LandRequests.add(rand.nextInt(numRqsts), (Object)obj);
		}
		
		//ilog.LogInfoLine("done.");

		// search through land requests and assign land
		// provide negative responses as necessary
		Pair land_rqst = null;
		Integer hhID;
		String land;
		Pair requester;
		AgentMsg msg;
		
		while ( !LandRequests.isEmpty() )
		{
			land_rqst = (Pair) LandRequests.removeFirst();
			hhID = (Integer) land_rqst.key;
			land = (String) land_rqst.value;
			String msg_key = TypeAgents.HOUSEHOLD.toString() + hhID.intValue() + Messages.GRAZINGPLAN.toString();
			String result = null;
			
			
			if ( !ManagedLand.containsKey(land) )
			{
				// assign land to this agent and send positive response
				requester = new Pair(TypeAgents.HOUSEHOLD, hhID);
				ManagedLand.put((String)land, (Pair)requester);
				result = "cultivated";
			}
			
			else
			{
				// check if the agent already manages the land
				// don't allow the agent to cultivate village-managed land or land managed by a different household
				requester = (Pair) ManagedLand.get(land);
				TypeAgents agent_type = (TypeAgents) requester.key;
				Integer agent_id = (Integer) requester.value;
				
				if ( (agent_type == TypeAgents.HOUSEHOLD) && (agent_id.intValue() == hhID.intValue()) )
					result = "cultivated";		// same household, send positive response
			
				else
					result = "managed";			// already managed, send negative response
			}
			
			//ilog.LogInfoLine("Request to GRAZE from Hh " + hhID + " for " + land + ". Response: " + result);
			
			// create/append message to agent
			if ( this.Responses.containsKey(msg_key) )
				msg = (AgentMsg) this.Responses.remove(msg_key);
			
			else
				msg = new AgentMsg(TypeAgents.HOUSEHOLD, hhID.intValue(), Messages.GRAZINGPLAN);
			
			msg.Add(new Pair(land, result));			
			this.Responses.put((String)msg_key, (AgentMsg)msg);
		}
		
		//ilog.LogInfoLine("Returning from ManageGrazingLandRqst()");
		
	}

}
