package PenaguilaALv4;

import java.util.Hashtable;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.NoSuchElementException;

/**
 * The OrderedHashtable class uses an ordered ArrayList to cross-reference hashtable
 * values The ArrayList stores (Number, Object) key pairs and uses the Number object
 * to sort the list in descending order of value. The Object is the hashtable key used
 * to reference the value stored in the hashtable.
 *
 * Note that the methods are not serialized.
 *
 * @author Gary R. Mayer
 *
 */
public class OrderedHashtable {
    private Hashtable<Object,Object> Hash = null;
    private ArrayList<KeyPair> List = null;    
    
    /**
     * Default constructor which provides a default initial size of 11
     * and a default load factor of 0.75.
     */
    public OrderedHashtable() {
        this(11, 0.75f);

    }

    /**
     * Constructor sets the initial capacity and provides a default load factor
     * of 0.75.
     * @param initialCapacity an integer value representing the initial
     *  size of the ordered hashtable.
     * @throws IllegalArgumentException if the initial capacity is less than zero.
     */
    public OrderedHashtable(int initialCapacity) {
        this(initialCapacity, 0.75f);

    }

    /**
     * Constructor sets the initial capacity and load factor for the
     * ordered hashtable.
     * @param initialCapacity an integer value representing the initial
     *  size of the ordered hashtable.
     * @param loadFactor a float value specifying the load factor for the
     * ordered hashtable.
     * @throws IllegalArgumentException if the initial capacity is less than
     * zero or load factor is non-positive.
     */
    public OrderedHashtable(int initialCapacity, float loadFactor) throws IllegalArgumentException {
        if ((initialCapacity < 0) || (loadFactor <= 0.0f)) {
            throw new IllegalArgumentException("The initial capacity must non-negative and load factor must be greater than zero");
        }
        
        this.Hash = new Hashtable<Object, Object>(initialCapacity, loadFactor);
        this.List = new ArrayList<KeyPair>(initialCapacity);

    }

    /**
     * Places an object into the ordered hashtable.
     *
     * @param sortValue a Number object used to sort the ordered hashtable.
     * @param hashKey an Object to use as a key for the ordered hashtable.
     * @param hashValue the value to store in the ordered hashtable with the
     * associated key
     * @return an Object representing the previous value of this key in the
     * hashtable or null if it did not have one.
     * @throws NullPointerException if the sort value, hash key, or hash value is null.
     */
    public Object put(Number sortValue, Object hashKey, Object hashValue) throws NullPointerException {
        if ((sortValue == null) || (hashKey == null) || (hashValue == null)) {
            throw new NullPointerException("The sort value, hash key, and hash value must not be null; unable to put objects into hashtable");
        }

        addToSortedList(sortValue, hashKey);
        Object prev_value = Hash.put(hashKey, hashValue);

        return prev_value;

    }

    /**
     * Removes all data from the ordered hashtable.
     */
    public void clear() {
        this.List.clear();
        this.Hash.clear();

    }

    /**
     * Returns the ordered hashtable value associated with the specified key.
     *
     * @param hashKey the Object to retrieve the value from in the ordered hashtable.
     * @return an Object representing the value in the ordered hashtable or null
     * if the hash key is not mapped to any value.
     * @throws NullPointerException if the hash key is null.
     */
    public Object getByHashKey(Object hashKey) throws NullPointerException {
        if (hashKey == null) {
            throw new NullPointerException("hash key is null; unable to retrieve hashtable value");
        }

        if (this.List.isEmpty()) {
            return null;
        }

        return this.Hash.get(hashKey);

    }

    /**
     * Returns the value(s) found with a matching sort value.
     *
     * @param sortValue the Number associated with the value(s) in the ordered hashtable.
     * @return an Object array containing all ordered hashtable values with the
     * specified sort value or null if no matching values are found in the ordered
     * hashtable.
     * @throws NullPointerException if sort value is null.
     */
    public Object[] getBySortValue(Number sortValue) throws NullPointerException {
        if (sortValue == null) {
            throw new NullPointerException("sort value is null; unable to retrieve hashtable values");
        }
        
        Object[] hashValues = null;
        Object[] hashKeys = null;
        
        try {
            hashKeys = this.getHashKeysFromSortValue(sortValue);

            if (hashKeys == null) {
                return null;
            }

            int num_keys = hashKeys.length;
            hashValues = new Object[num_keys];

            for (int hv = 0; hv < num_keys; hv++) {
                if (hashKeys[hv] == null) {
                    throw new NullPointerException("hash key retrieved from sort value is null; unable to retrieve hashtable values");
                }
                
                hashValues[hv] = this.getByHashKey(hashKeys[hv]);
            }

            return hashValues;
        }
        
        finally {
            hashValues = null;
            hashKeys = null;
        }

    }

    /**
     * Returns the ordered hashtable value specified by the index.
     *
     * @param index the integer referencing a value in the ordered hashtable.
     * @return an Object representing the ordered hashtable value at the specified
     * index.
     * 
     * Note that the hashtable is ordered in descending order of value. Thus, index
     * 0 will retrieve the ordered hashtable value with the highest sort value.
     * @throws IndexOutOfBoundsException if (index < 0) or (index >= size()).
     */
    public Object getByIndex(int index) throws IndexOutOfBoundsException {
        if ((index < 0) || (index >= this.List.size())) {
            throw new IndexOutOfBoundsException("index value out of range for current ordered hashtable content");
        }

        Object key = getHashKeyFromIndex(index);

        return this.getByHashKey(key);

    }

    /**
     * Returns the ordered hashtable value with the highest sort value.
     *
     * @return an Object representing the first value stored in the ordered
     * hashtable or null if the ordered hashtable is empty.
     * 
     * Note that if there are multiple hash values with the same sort value, only
     * the first value is being returned. This is equivalent to getByIndex(0),
     * except that this function does not throw an exception if the ordered hashtable
     * is empty.
     */
    public Object getHighest() {        
        try {
            return this.getByIndex(0);
        }
        
        catch (IndexOutOfBoundsException ioobe) {
            return null;
        }

    }

    /**
     * Returns the ordered hashtable value with the lowest sort value.
     *
     * @return an Object representing the last value stored in the ordered
     * hashtable or null if the ordered hashtable is empty.
     * 
     * Note that if there are multiple hash values with the same sort value, only
     * the last value is being returned. This is equivalent to getByIndex(size() - 1),
     * except that this function does not throw an exception if the ordered hashtable
     * is empty.
     */
    public Object getLowest() {
        Object lowest_value = null;
        
        try {
            lowest_value = this.getByIndex(this.List.size() - 1);
        }
        
        catch (IndexOutOfBoundsException ioobe) {
            lowest_value = null;
        }
        
        return lowest_value;

    }

    /**
     * Returns the value stored in the ordered hashtable represented by the
     * median sort value.
     *
     * @return an Object representing the value associated with the median
     * sort value value or null if the ordered hashtable is empty.
     * 
     * Note that if there are an odd number of elements in the ordered hashtable,
     * then this is the middle value. However, if there are an even number, then
     * the returned value is biased towards the higher-valued element as the mean
     * of the two closest elements in the ordered hashtable has no meaning. This
     * function is equivalent to getByIndex((size() - 1) / 2), except that this
     * function does not throw an exception if the ordered hashtable is empty.
     */
    public Object getMedian() {
        int median_index = (this.List.size() - 1) / 2;
        Object median_key = getHashKeyFromIndex(median_index);

        if (median_key == null) {
            return null;
        }

        return this.getByHashKey(median_key);

    }

    /**
     * Returns the highest sort value in the ordered hashtable.
     *
     * @return a Number representing the highest sort value or null if the ordered
     * hashtable is empty.
     */
    public Number getHighestSortValue() {
        KeyPair max_pair = null;
        
        try {
            max_pair = this.List.get(0);
            return max_pair.sortValue;
        }
        
        catch (IndexOutOfBoundsException ioobe) {
            return null;
        }

    }

    /**
     * Returns the lowest sort value in the ordered hashtable.
     *
     * @return a Number representing the lowest sort value or null if the ordered
     * hashtable is empty.
     */
    public Number getLowestSortValue() {
        KeyPair min_pair = null;
        
        try {
            min_pair = this.List.get(this.List.size() - 1);
            return min_pair.sortValue;
        }
        
        catch (IndexOutOfBoundsException ioobe) {
            return null;
        }

    }

    /**
     * Returns the median sort value in the ordered hashtable.
     *
     * @return a Number representing the median sort value or null if the ordered
     * hashtable is empty.
     * Note that if there is an odd number of elements in the ordered hashtable,
     * then this is the middle value. However, if there is an even number, then
     * the returned value is biased towards the higher-valued element as the mean
     * of the two closest elements in the ordered hashtable has no meaning.
     */
    public Number getMedianSortValue() {
        int median_index = (this.List.size() - 1) / 2;
        KeyPair median_pair = null;
        
        try {
            median_pair = this.List.get(median_index);
            return median_pair.sortValue;
        }
        
        catch (IndexOutOfBoundsException ioobe) {
            return null;
        }
        
    }

    /**
     * Returns true if the ordered hashtable contains the specified sort value.
     *
     * @param sortValue a Number representing the value to search for in the hashtable.
     * @return true if the sort value exists at least once; false otherwise.
     * @throws NullPointerException if the sort value is null.
     */
    public boolean containsSortValue(Number sortValue) throws NullPointerException {
        if (sortValue == null) {
            throw new NullPointerException("sort value is null; unable to query ordered hashtable");
        }
        
        Object[] hashKeys = this.getHashKeysFromSortValue(sortValue);

        if (hashKeys == null) {
            return false;
        }
        
        else {
            return true;
        }

    }

    /**
     * Returns true if the ordered hashtable contains the specified hash key.
     *
     * @param hashKey an Object representing the key to search for in the ordered
     * hashtable.
     * @return true if the key exists; false otherwise.
     * @throws NullPointerException if the hash key is null.
     */
    public boolean containsHashKey(Object hashKey) throws NullPointerException {
        if (hashKey == null) {
            throw new NullPointerException("hash key is null; unable to query ordered hashtable");
        }
        
        return this.Hash.containsKey(hashKey);

    }

    /**
     * Returns true if the ordered hashtable contains the specified hash value.
     *
     * @param hashValue an Object representing the hash value to search for in the
     * ordered hashtable.
     * @return true if the hash value exists; false otherwise.
     * @throws NullPointerException if the hash value is null.
     */
    public boolean containsHashValue(Object hashValue) throws NullPointerException {
        if (hashValue == null) {
            throw new NullPointerException("hash value is null; unable to query ordered hashtable");
        }
        
        return this.Hash.containsValue(hashValue);

    }
    
    /**
     * Removes the hash key (and its corresponding hash value and sort value) from
     * this hashtable. This method does nothing if the hash key is not in the ordered
     * hashtable. Note that if the hashtable contains a large number of elements, this is
     * a very costly operation as it must search through the sort values linearly --
     * i.e., in O(n) time.
     * 
     * @param hashKey an Object specifying the hash key to be removed.
     * @return an Object representing the hash value to which the hash key had been
     * mapped in the ordered hashtable or null if the key did not have a mapping.
     * @throws NullPointerException if the hash key is null.
     * @throws IllegalArgumentException if the hash key is found in the ordered hashtable
     * but the sort value is not. Note that the hash value will not be removed from the
     * ordered hashtable if both the hash key and the sort value are not found.
     */
    public Object remove(Object hashKey) throws NullPointerException,
    											IllegalArgumentException {
        if (hashKey == null) {
            throw new NullPointerException("hash key is null; no elements removed from ordered hashtable");
        }
        
        boolean key_exists = this.Hash.containsKey(hashKey);
        
        if (!key_exists) {
        	return null;
        }
        
        // search through array list and remove associated search value
        int num_keypairs = this.List.size();
        KeyPair curr_pair = null;
        
        int kp;
        for (kp = 0; kp < num_keypairs; kp++) {
            curr_pair = this.List.get(kp);
            
            if (curr_pair.hashKey.equals(hashKey)) {
                this.List.remove(kp);
                break;
            }
        }
        
        if (kp == num_keypairs) {
        	String err_msg = "An element was found in the ordered hashtable with the " +
        					 "specified hash key but the specified sort value could not " + 
        					 "be found. No elements were removed from the ordered hashtable.\n" +
        					 "hash key: " + hashKey.toString();
            throw new IllegalArgumentException(err_msg);
        }
        
        return this.Hash.remove(hashKey);
        
    }
    
    /**
     * Removes the hash key (and its corresponding hash value and sort value) from
     * this hashtable. This method does nothing if the hash key is not in the ordered
     * hashtable. The sort value removal operates in approximately O(log n) time.
     * 
     * @param sortValue a Number representing the sort value in the ordered hashtable.
     * @param hashKey an Object representing the hash key in the ordered hashtable.
     * @return an Object representing the ordered hashtable value removed or null if
     * the hash key was not found in the ordered hashtable. 
     * @throws NullPointerException if either the sort value or hash key are null.
     * @throws IllegalArgumentException if the hash key is found in the ordered hashtable
     * but the sort value is not. Note that the hash value will not be removed from the
     * ordered hashtable if both the hash key and the sort value are not found.
     */
    public Object remove(Number sortValue, Object hashKey) throws NullPointerException,
                                                                  IllegalArgumentException {
        if ((sortValue == null) || (hashKey == null)) {
            throw new NullPointerException("sort value and hash key values must not be null; no elements removed from ordered hahstable");
        }
        
        boolean key_exists = this.Hash.containsKey(hashKey);
        
        if (!key_exists) {
        	return null;
        }
        
        int remove_index = getIndexFromSortValueAndHashKey(sortValue, hashKey);
        
        if (remove_index == -1) {
        	String err_msg = "An element was found in the ordered hashtable with the " +
			 				 "specified hash key but the specified sort value could not " + 
			 				 "be found. No elements were removed from the ordered hashtable.\n" +
			 				 "hash key: " + hashKey.toString() + ", sort value: " + sortValue.toString();
        	throw new IllegalArgumentException(err_msg);       }
        
        else {
        	this.List.remove(remove_index);
        }
/*
        boolean removed = this.List.remove(new KeyPair(sortValue, hashKey));
        
        if  (!removed) {
            throw new IllegalArgumentException("An element was found and removed from the ordered hashtable but the specified sort value could not be found.");
        }
*/
        return this.Hash.remove(hashKey);
        
    }
    
    /**
     * Removes the hash value in the ordered hashtable with the highest sort value.
     * 
     * @return an Object representing the removed ordered hashtable element or null
     * if the ordered hashtable is empty.
     */
    public Object removeHighest() {
    	if (this.List.isEmpty()) {
    		return null;
    	}
    	
    	KeyPair first_pair = this.List.remove(0);
    	
    	return this.Hash.remove(first_pair.hashKey);
    	
    }
    
    /**
     * Removes the hash value in the ordered hashtable with the lowest sort value.
     * 
     * @return an Object representing the removed ordered hashtable element or null
     * if the ordered hashtable is empty.
     */public Object removeLowest() {
    	if (this.List.isEmpty()) {
    		return null;
    	}
    	
    	KeyPair last_pair = this.List.remove(this.List.size() - 1);
    	
    	return this.Hash.remove(last_pair.hashKey);
    	
    }

    /**
     * Returns the number of entries in the ordered hashtable.
     *
     * @return an integer value representing the number of entries in the
     * ordered hashtable.
     */
    public int size() {
        return this.List.size();

    }

    /**
     * Returns true if there are no elements in the ordered hashtable.
     *
     * @return a boolean value that is true if there are no elements in the ordered
     * hashtable; false otherwise.
     */
    public boolean isEmpty() {
        return this.List.isEmpty();

    }
    
    /**
     * Returns an enumeration of the hash keys in the ordered hashtable.
     * Note that these keys are NOT ordered by the associated sort values.
     * 
     * @return an Enumeration object containing the hash keys in the ordered hashtable.
     */
    public Enumeration keys () {
        return this.Hash.keys();
        
    }
    
    /**
     * Returns an enumeration of the hash values in the ordered hashtable.
     * Note that these values are NOT ordered by the associated sort values.
     * 
     * @return an Enumeration object containing the ordered hashtable hash values.
     */
    public Enumeration elements () {
        return this.Hash.elements();
        
    }
    
    public Enumeration keysInOrder() {
    	return new Enumeration() {
    		private int curr_index = 0;
    		private int num_elements = List.size();
    		
    		/**
    		 * Returns true if the enumeration has more elements.
    		 * 
    		 * @return true if there are more elements in the enumeration; false otherwise.
    		 */
    		public boolean hasMoreElements () {
    			return curr_index < num_elements;
    		}
    		
    		/**
    		 * Gets the next hash key in the enumeration.
    		 * 
    		 * @return an Object representing the next hash key in the ordered hashtable. 
    		 * @throws NoSuchElementException if there are no remaining hash keys in the enumeration.
    		 */
    		public Object nextElement() throws NoSuchElementException {
    			int return_index = curr_index;
    			++curr_index;
    			
    			if (return_index >= num_elements) {
    				throw new NoSuchElementException("Enumeration contains no remainign elements.");
    			}
    			
    			return (List.get(return_index)).hashKey;
    		}
    	};
    	
    }
    
    public Enumeration elementsInOrder() {
    	return new Enumeration() {
    		private int curr_index = 0;
    		private int num_elements = List.size();
    		
    		/**
    		 * Returns true if the enumeration has more elements.
    		 * 
    		 * @return true if there are more elements in the enumeration; false otherwise.
    		 */
    		public boolean hasMoreElements () {
    			return curr_index < num_elements;
    		}
    		
    		/**
    		 * Gets the next hash value in the enumeration.
    		 * 
    		 * @return an Object representing the next hash value in the ordered hashtable. 
    		 * @throws NoSuchElementException if there are no remaining hash values in the enumeration.
    		 */
    		public Object nextElement() throws NoSuchElementException {
    			int return_index = curr_index;
    			++curr_index;
    			
    			if (return_index >= num_elements) {
    				throw new NoSuchElementException("Enumeration contains no remainign elements.");
    			}
    			
    			return getByIndex(return_index);
    		}
    	};
    	
    }
    
    /**
     * Returns a string representation of the ordered hashtable. Each element in the hashtable
     * is appended to the string in sort value descending order as a comma separated, 
     * [sort_value, hash_key, hash_value] triplet.
     * 
     * @return the String representation of this ordered hashtable.
     */
    public String toString() {
    	String obj_string = new String();
    	
    	KeyPair curr_pair = null;
    	Object curr_hvalue = null;
    	int num_elements = this.List.size();
    	
    	for (int e = 0; e < num_elements; e++) {
    		curr_pair = this.List.get(e);
    		curr_hvalue = this.Hash.get(curr_pair.hashKey);
    		
    		if (e > 0) {
    			obj_string += ", ";
    		}
    		
    		obj_string += "[" + curr_pair.sortValue.toString() + ", " + curr_pair.hashKey.toString() + ", " + curr_hvalue.toString() + "]";
    	}
    	
    	return obj_string;
    }

    /**
     * Adds a KeyPair object to the ordered hashtable's sorted list.
     * 
     * @param sortValue
     * @param hashKey
     */
    private void addToSortedList(Number sortValue, Object hashKey) {
        if ((sortValue == null) || (hashKey == null)) {
            return;
        }
        
        KeyPair new_pair = new KeyPair(sortValue, hashKey);
        
        if (this.List.size() == 0) {
            this.List.add(new_pair);
        }
        
        else {
            Number curr_key = null;
            int min_index = 0;
            int max_index = this.List.size();
            int curr_index = -1;
            boolean found = false;

            while (min_index < max_index) {
                curr_index = (min_index + max_index) / 2;
                curr_key = (this.List.get(curr_index)).sortValue;

                if (sortValue.doubleValue() > curr_key.doubleValue()) {
                    max_index = curr_index;
                } else if (sortValue.doubleValue() < curr_key.doubleValue()) {
                    min_index = curr_index + 1;
                } else {
                    found = true;
                    break;
                }
            }
            
            // if found, just add it here
            if (found) {
                this.List.add(curr_index, new_pair);
            }
            
            else {
                // min_index = size() if sort value is smaller than existing values
                if (min_index == this.List.size()) {
                    this.List.add(new_pair);
                }
                
                // min_index and max_index converge on insertion index
                else {
                    this.List.add(min_index, new_pair);
                }
            }
        }

    }

    /**
     * Returns the hash key associated with the specified index.
     * 
     * @param index
     * @return
     */
    private Object getHashKeyFromIndex(int index) {
        if ((index < 0) || (index >= this.List.size())) {
            return null;
        }

        KeyPair pair = this.List.get(index);

        return pair.hashKey;

    }

    /**
     * Returns an array of hash keys associated with a specific sort value.
     * 
     * @param sortValue
     * @return
     */
    private Object[] getHashKeysFromSortValue(Number sortValue) {
        if (sortValue == null) {
            return null;
        }
        
        if (this.List.size() == 0) {
            return null;
        }
        
        Object[] keys = null;
        Number curr_key = null;
        int min_index = 0;
        int max_index = this.List.size();
        int curr_index = -1;
        boolean found = false;

        while (min_index < max_index) {
            curr_index = (min_index + max_index) / 2;
            curr_key = (this.List.get(curr_index)).sortValue;

            if (sortValue.doubleValue() > curr_key.doubleValue()) {
                max_index = curr_index;
            } else if (sortValue.doubleValue() < curr_key.doubleValue()) {
                min_index = curr_index + 1;
            } else {
                found = true;
                break;
            }
        }

        if (!found) {
            return null;
        }

        // search back and forward to create array
        int low_index, high_index;
        int step = -1;
        while (curr_index + step > -1) {
            curr_key = (this.List.get(curr_index + step)).sortValue;

            if (sortValue.doubleValue() != curr_key.doubleValue()) {
                break;
            }

            --step;
        }

        low_index = curr_index + step + 1;

        step = 1;
        while (curr_index + step < this.List.size()) {
            curr_key = (this.List.get(curr_index + step)).sortValue;

            if (sortValue.doubleValue() != curr_key.doubleValue()) {
                break;
            }

            ++step;
        }

        high_index = curr_index + step - 1;

        keys = new Object[high_index - low_index + 1];

        for (int i = low_index; i <= high_index; i++) {
        	KeyPair key_pair = this.List.get(i);
            keys[i - low_index] = key_pair.hashKey;
        }

        return keys;

    }
    
    private int getIndexFromSortValueAndHashKey (Number sortValue, Object hashKey) {
    	if ((sortValue == null) || (hashKey == null)) {
    		return -1;
    	}
    	
        if (this.List.size() == 0) {
            return -1;
        }
        
        KeyPair curr_pair = null;
        Number curr_value = null;
        int min_index = 0;
        int max_index = this.List.size();
        int curr_index = -1;
        boolean found = false;

        while (min_index < max_index) {
            curr_index = (min_index + max_index) / 2;
            curr_pair = this.List.get(curr_index);
            curr_value = curr_pair.sortValue;

            if (sortValue.doubleValue() > curr_value.doubleValue()) {
                max_index = curr_index;
            } else if (sortValue.doubleValue() < curr_value.doubleValue()) {
                min_index = curr_index + 1;
            } else {
                found = true;
                break;
            }
        }
        
        // a matching sort value was found; check hash key for match
        if (found) {
        	if (hashKey.equals(curr_pair.hashKey)) {
        		return curr_index;
        	}
        	
        	// walk along like valued elements to find hashKey
        	else {
                int low_index, high_index;
                int step = -1;
                while (curr_index + step > -1) {
                    curr_value = (this.List.get(curr_index + step)).sortValue;

                    if (sortValue.doubleValue() != curr_value.doubleValue()) {
                        break;
                    }

                    --step;
                }

                low_index = curr_index + step + 1;

                step = 1;
                while (curr_index + step < this.List.size()) {
                    curr_value = (this.List.get(curr_index + step)).sortValue;

                    if (sortValue.doubleValue() != curr_value.doubleValue()) {
                        break;
                    }

                    ++step;
                }

                high_index = curr_index + step - 1;

                // search through all matching values for hash key; if found, return its index
                for (int i = low_index; i <= high_index; i++) {
                    curr_pair = this.List.get(i);
                    
                    if (curr_pair.hashKey.equals(hashKey)) {
                    	return i;
                    }
                }
                
                // not found amoung matching values
                return -1;
        	}
        }
        
        // value not found
        else {
            return -1;
        }
        
    }
    
    /**
     * A convenience class used to map a sort value to a hash key. The key pair
     * objects are stored in the ordered hashtable array list in descending order
     * by the sort value.
     */
    private class KeyPair {

        Number sortValue = null;
        Object hashKey = null;

        /**
         * COnstructor creates a new key pair object.
         * @param sortValue a Number representing the sort value
         * @param hashKey an Object representing the hash key to map to the sort value
         */
        public KeyPair(Number sortValue, Object hashKey) {
            this.sortValue = sortValue;
            this.hashKey = hashKey;

        }
        
        private KeyPair () {
        }
    }
}
